/// autogenerated analytical inverse kinematics code from ikfast program
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// generated 2010-07-18 18:37:38.232097
/// To compile with gcc use: gcc -lstdc++ 
/// To compile without any main function use: gcc -lstdc++ -DIKFAST_NO_MAIN
#include <math.h>
#include <assert.h>
#include <vector>
#include <float.h>

#define IK2PI  6.28318530717959
#define IKPI  3.14159265358979
#define IKPI_2  1.57079632679490

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// defined when creating a shared object/dll
#ifdef IKFAST_CLIBRARY
#ifdef _MSC_VER
#define IKFAST_API extern "C" __declspec(dllexport)
#else
#define IKFAST_API extern "C"
#endif
#else
#define IKFAST_API
#endif

typedef double IKReal;
class IKSolution
{
public:
    /// Gets a solution given its free parameters
    /// \param pfree The free parameters required, range is in [-pi,pi]
    void GetSolution(IKReal* psolution, const IKReal* pfree) const {
        for(size_t i = 0; i < basesol.size(); ++i) {
            if( basesol[i].freeind < 0 )
                psolution[i] = basesol[i].foffset;
            else {
                assert(pfree != NULL);
                psolution[i] = pfree[basesol[i].freeind]*basesol[i].fmul + basesol[i].foffset;
                if( psolution[i] > IKPI )
                    psolution[i] -= IK2PI;
                else if( psolution[i] < -IKPI )
                    psolution[i] += IK2PI;
            }
        }
    }

    /// Gets the free parameters the solution requires to be set before a full solution can be returned
    /// \return vector of indices indicating the free parameters
    const std::vector<int>& GetFree() const { return vfree; }

    struct VARIABLE
    {
        VARIABLE() : freeind(-1), fmul(0), foffset(0) {}
        VARIABLE(int freeind, IKReal fmul, IKReal foffset) : freeind(freeind), fmul(fmul), foffset(foffset) {}
        int freeind;
        IKReal fmul, foffset; ///< joint value is fmul*sol[freeind]+foffset
    };

    std::vector<VARIABLE> basesol;       ///< solution and their offsets if joints are mimiced
    std::vector<int> vfree;
};

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKasin(float f)
{
assert( f > -1.001f && f < 1.001f ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asinf(f);
}
inline double IKasin(double f)
{
assert( f > -1.001 && f < 1.001 ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

inline float IKacos(float f)
{
assert( f > -1.001f && f < 1.001f ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0.0f;
return acosf(f);
}
inline double IKacos(double f)
{
assert( f > -1.001 && f < 1.001 ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0.0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        assert(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) )
        return 0;
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        assert(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) )
        return 0;
    return atan2(fy,fx);
}

IKFAST_API int getNumFreeParameters() { return 1; }
IKFAST_API int* getFreeParameters() { static int freeparams[] = {6}; return freeparams; }
IKFAST_API int getNumJoints() { return 7; }

IKFAST_API int getIKRealSize() { return sizeof(IKReal); }

IKFAST_API int getIKType() { return 1; }

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void fk(const IKReal* j, IKReal* eetrans, IKReal* eerot) {
    IKReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13;
    x0=IKcos(j[0]);
    x1=IKcos(j[3]);
    x2=IKcos(j[5]);
    x3=IKsin(j[6]);
    x4=IKcos(j[2]);
    x5=IKcos(j[4]);
    x6=IKcos(j[6]);
    x7=IKsin(j[0]);
    x8=IKsin(j[4]);
    x9=IKsin(j[5]);
    x10=IKsin(j[1]);
    x11=IKsin(j[2]);
    x12=IKsin(j[3]);
    x13=IKcos(j[1]);
    eerot[0]=((((-1.00000000000000)*(x0)*(x12)*(x4)*(x6)*(x9)))+(((-1.00000000000000)*(x1)*(x10)*(x11)*(x3)*(x7)*(x8)*(x9)))+(((x0)*(x1)*(x3)*(x4)*(x8)*(x9)))+(((-1.00000000000000)*(x12)*(x13)*(x2)*(x6)*(x7)*(x8)))+(((-1.00000000000000)*(x1)*(x13)*(x6)*(x7)*(x9)))+(((-1.00000000000000)*(x12)*(x13)*(x3)*(x7)*(x8)*(x9)))+(((x0)*(x12)*(x2)*(x3)*(x4)))+(((-1.00000000000000)*(x10)*(x11)*(x12)*(x2)*(x3)*(x7)))+(((x0)*(x11)*(x3)*(x5)*(x9)))+(((x10)*(x2)*(x4)*(x5)*(x6)*(x7)))+(((-1.00000000000000)*(x1)*(x10)*(x11)*(x2)*(x6)*(x7)*(x8)))+(((x10)*(x3)*(x4)*(x5)*(x7)*(x9)))+(((x0)*(x11)*(x2)*(x5)*(x6)))+(((x1)*(x13)*(x2)*(x3)*(x7)))+(((x10)*(x11)*(x12)*(x6)*(x7)*(x9)))+(((x0)*(x1)*(x2)*(x4)*(x6)*(x8))));
    eerot[1]=((((x1)*(x11)*(x13)*(x2)*(x6)*(x8)))+(((-1.00000000000000)*(x13)*(x3)*(x4)*(x5)*(x9)))+(((x11)*(x12)*(x13)*(x2)*(x3)))+(((-1.00000000000000)*(x1)*(x10)*(x6)*(x9)))+(((-1.00000000000000)*(x11)*(x12)*(x13)*(x6)*(x9)))+(((-1.00000000000000)*(x13)*(x2)*(x4)*(x5)*(x6)))+(((-1.00000000000000)*(x10)*(x12)*(x2)*(x6)*(x8)))+(((-1.00000000000000)*(x10)*(x12)*(x3)*(x8)*(x9)))+(((x1)*(x11)*(x13)*(x3)*(x8)*(x9)))+(((x1)*(x10)*(x2)*(x3))));
    eerot[2]=((((x0)*(x10)*(x11)*(x12)*(x2)*(x3)))+(((x0)*(x1)*(x10)*(x11)*(x2)*(x6)*(x8)))+(((-1.00000000000000)*(x0)*(x1)*(x13)*(x2)*(x3)))+(((x1)*(x3)*(x4)*(x7)*(x8)*(x9)))+(((x0)*(x12)*(x13)*(x2)*(x6)*(x8)))+(((x0)*(x12)*(x13)*(x3)*(x8)*(x9)))+(((-1.00000000000000)*(x12)*(x4)*(x6)*(x7)*(x9)))+(((-1.00000000000000)*(x0)*(x10)*(x11)*(x12)*(x6)*(x9)))+(((-1.00000000000000)*(x0)*(x10)*(x3)*(x4)*(x5)*(x9)))+(((x1)*(x2)*(x4)*(x6)*(x7)*(x8)))+(((x12)*(x2)*(x3)*(x4)*(x7)))+(((x11)*(x2)*(x5)*(x6)*(x7)))+(((x0)*(x1)*(x10)*(x11)*(x3)*(x8)*(x9)))+(((x11)*(x3)*(x5)*(x7)*(x9)))+(((x0)*(x1)*(x13)*(x6)*(x9)))+(((-1.00000000000000)*(x0)*(x10)*(x2)*(x4)*(x5)*(x6))));
    eetrans[0]=((((-0.0329999999999018)*(x3)))+(((0.149999999999636)*(x1)*(x6)*(x9)))+(((0.149999999999636)*(x12)*(x2)*(x6)*(x8)))+(((-0.0140000000000100)*(x6)))+(((0.149999999999636)*(x12)*(x3)*(x8)*(x9)))+(((0.155000000000655)*(x6)*(x9)))+(((-0.155000000000655)*(x2)*(x3)))+(((-0.149999999999636)*(x1)*(x2)*(x3))));
    eerot[3]=((((-1.00000000000000)*(x12)*(x13)*(x5)*(x7)))+(((-1.00000000000000)*(x0)*(x11)*(x8)))+(((-1.00000000000000)*(x10)*(x4)*(x7)*(x8)))+(((x0)*(x1)*(x4)*(x5)))+(((-1.00000000000000)*(x1)*(x10)*(x11)*(x5)*(x7))));
    eerot[4]=((((-1.00000000000000)*(x10)*(x12)*(x5)))+(((x1)*(x11)*(x13)*(x5)))+(((x13)*(x4)*(x8))));
    eerot[5]=((((x1)*(x4)*(x5)*(x7)))+(((-1.00000000000000)*(x11)*(x7)*(x8)))+(((x0)*(x12)*(x13)*(x5)))+(((x0)*(x10)*(x4)*(x8)))+(((x0)*(x1)*(x10)*(x11)*(x5))));
    eetrans[1]=((0.160000000000000)+(((0.149999999999636)*(x12)*(x5))));
    eerot[6]=((((-1.00000000000000)*(x0)*(x12)*(x2)*(x4)*(x6)))+(((-1.00000000000000)*(x12)*(x13)*(x2)*(x3)*(x7)*(x8)))+(((x10)*(x11)*(x12)*(x3)*(x7)*(x9)))+(((-1.00000000000000)*(x10)*(x4)*(x5)*(x6)*(x7)*(x9)))+(((x12)*(x13)*(x6)*(x7)*(x8)*(x9)))+(((x0)*(x1)*(x2)*(x3)*(x4)*(x8)))+(((-1.00000000000000)*(x1)*(x10)*(x11)*(x2)*(x3)*(x7)*(x8)))+(((x10)*(x11)*(x12)*(x2)*(x6)*(x7)))+(((-1.00000000000000)*(x0)*(x12)*(x3)*(x4)*(x9)))+(((-1.00000000000000)*(x1)*(x13)*(x3)*(x7)*(x9)))+(((x10)*(x2)*(x3)*(x4)*(x5)*(x7)))+(((-1.00000000000000)*(x0)*(x1)*(x4)*(x6)*(x8)*(x9)))+(((-1.00000000000000)*(x0)*(x11)*(x5)*(x6)*(x9)))+(((-1.00000000000000)*(x1)*(x13)*(x2)*(x6)*(x7)))+(((x0)*(x11)*(x2)*(x3)*(x5)))+(((x1)*(x10)*(x11)*(x6)*(x7)*(x8)*(x9))));
    eerot[7]=((((x10)*(x12)*(x6)*(x8)*(x9)))+(((-1.00000000000000)*(x1)*(x10)*(x3)*(x9)))+(((-1.00000000000000)*(x1)*(x10)*(x2)*(x6)))+(((-1.00000000000000)*(x11)*(x12)*(x13)*(x3)*(x9)))+(((-1.00000000000000)*(x13)*(x2)*(x3)*(x4)*(x5)))+(((-1.00000000000000)*(x11)*(x12)*(x13)*(x2)*(x6)))+(((x13)*(x4)*(x5)*(x6)*(x9)))+(((x1)*(x11)*(x13)*(x2)*(x3)*(x8)))+(((-1.00000000000000)*(x1)*(x11)*(x13)*(x6)*(x8)*(x9)))+(((-1.00000000000000)*(x10)*(x12)*(x2)*(x3)*(x8))));
    eerot[8]=((((-1.00000000000000)*(x12)*(x2)*(x4)*(x6)*(x7)))+(((x0)*(x1)*(x13)*(x2)*(x6)))+(((x0)*(x1)*(x10)*(x11)*(x2)*(x3)*(x8)))+(((-1.00000000000000)*(x0)*(x1)*(x10)*(x11)*(x6)*(x8)*(x9)))+(((x1)*(x2)*(x3)*(x4)*(x7)*(x8)))+(((-1.00000000000000)*(x0)*(x10)*(x11)*(x12)*(x3)*(x9)))+(((x0)*(x10)*(x4)*(x5)*(x6)*(x9)))+(((-1.00000000000000)*(x0)*(x12)*(x13)*(x6)*(x8)*(x9)))+(((-1.00000000000000)*(x0)*(x10)*(x2)*(x3)*(x4)*(x5)))+(((x11)*(x2)*(x3)*(x5)*(x7)))+(((-1.00000000000000)*(x12)*(x3)*(x4)*(x7)*(x9)))+(((x0)*(x1)*(x13)*(x3)*(x9)))+(((-1.00000000000000)*(x0)*(x10)*(x11)*(x12)*(x2)*(x6)))+(((-1.00000000000000)*(x1)*(x4)*(x6)*(x7)*(x8)*(x9)))+(((x0)*(x12)*(x13)*(x2)*(x3)*(x8)))+(((-1.00000000000000)*(x11)*(x5)*(x6)*(x7)*(x9))));
    eetrans[2]=((((0.155000000000655)*(x2)*(x6)))+(((0.0329999999999018)*(x6)))+(((-0.0140000000000100)*(x3)))+(((0.149999999999636)*(x1)*(x3)*(x9)))+(((-0.149999999999636)*(x12)*(x6)*(x8)*(x9)))+(((0.155000000000655)*(x3)*(x9)))+(((0.149999999999636)*(x12)*(x2)*(x3)*(x8)))+(((0.149999999999636)*(x1)*(x2)*(x6))));
    }

/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ik(const IKReal* eetrans, const IKReal* eerot, const IKReal* pfree, std::vector<IKSolution>& vsolutions) {
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    vsolutions.resize(0); vsolutions.reserve(8);
    IKReal j0, cj0, sj0,
    j1, cj1, sj1,
    j2, cj2, sj2,
    j3, cj3, sj3,
    j4, cj4, sj4,
    j5, cj5, sj5,
    j6=pfree[0], cj6=cos(pfree[0]), sj6=sin(pfree[0]),
    new_r00, r00 = eerot[0*3+0],
    new_r01, r01 = eerot[0*3+1],
    new_r02, r02 = eerot[0*3+2],
    new_r10, r10 = eerot[1*3+0],
    new_r11, r11 = eerot[1*3+1],
    new_r12, r12 = eerot[1*3+2],
    new_r20, r20 = eerot[2*3+0],
    new_r21, r21 = eerot[2*3+1],
    new_r22, r22 = eerot[2*3+2],
    new_px, new_py, new_pz, px = eetrans[0], py = eetrans[1], pz = eetrans[2];
    
    new_r00=r00;
    new_r01=r10;
    new_r02=r20;
    new_px=((((-1.00000000000000)*(px)*(r00)))+(((0.160000000000000)*(r10)))+(((-1.00000000000000)*(pz)*(r20)))+(((-1.00000000000000)*(py)*(r10))));
    new_r10=r01;
    new_r11=r11;
    new_r12=r21;
    new_py=((((0.160000000000000)*(r11)))+(((-1.00000000000000)*(py)*(r11)))+(((-1.00000000000000)*(px)*(r01)))+(((-1.00000000000000)*(pz)*(r21))));
    new_r20=r02;
    new_r21=r12;
    new_r22=r22;
    new_pz=((((-1.00000000000000)*(px)*(r02)))+(((-1.00000000000000)*(py)*(r12)))+(((0.160000000000000)*(r12)))+(((-1.00000000000000)*(pz)*(r22))));
    r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
    {
    IKReal evalcond;
        evalcond=j6;
        if( IKabs(evalcond) < 0.00001 )
        {
        {
        IKReal j3array[2], cj3array[2], sj3array[2];
        bool j3valid[2]={false};
        cj3array[0]=((-0.972903225803748)+(((-1.41935483871202)*(pz)))+(((21.5053763440810)*((py)*(py))))+(((21.5053763440810)*((pz)*(pz))))+(((0.602150537633861)*(px)))+(((21.5053763440810)*((px)*(px)))));
        if( cj3array[0] >= -1.0001 && cj3array[0] <= 1.0001 )
        {
            j3valid[0] = j3valid[1] = true;
            j3array[0] = IKacos(cj3array[0]);
            sj3array[0] = IKsin(j3array[0]);
            cj3array[1] = cj3array[0];
            j3array[1] = -j3array[0];
            sj3array[1] = -sj3array[0];
        }
        else if( isnan(cj3array[0]) )
        {
            // probably any value will work
            j3valid[0] = true;
            cj3array[0] = 1; sj3array[0] = 0; j3array[0] = 0;
        }
        if( j3valid[0] && j3valid[1] && IKabs(cj3array[0]-cj3array[1]) < 0.0001 && IKabs(sj3array[0]-sj3array[1]) < 0.0001 )
            j3valid[1]=false;
        for(int ij3 = 0; ij3 < 2; ++ij3)
        {
        if( !j3valid[ij3] ) { continue; }
        j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
        
        {
        IKReal evalcond[3]; int numsolutionsj4 = 0;
        IKReal j4array[2], cj4array[2], sj4array[2];
        bool j4valid[2]={false};
            {
            evalcond[0]=sj3;
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                cj4array[0]=((6.66666666668607)*(py)*(((IKabs(sj3) != 0)?((IKReal)1/(sj3)):(IKReal)1.0e30)));
                if( cj4array[0] >= -1.0001 && cj4array[0] <= 1.0001 )
                {
                    j4valid[0] = j4valid[1] = true;
                    j4array[0] = IKacos(cj4array[0]);
                    sj4array[0] = IKsin(j4array[0]);
                    cj4array[1] = cj4array[0];
                    j4array[1] = -j4array[0];
                    sj4array[1] = -sj4array[0];
                }
                else if( isnan(cj4array[0]) )
                {
                    // probably any value will work
                    j4valid[0] = true;
                    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
                }
                numsolutionsj4 = 2;
                
            } else
            {
            IKReal x0=((0.149999999999636)*(sj3));
            IKReal x1=((((-1.00000000000000)*(py)))+(x0));
            evalcond[0]=x1;
            evalcond[1]=((0.0452399999999216)+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-0.0280000000000200)*(px)))+(((-1.00000000000000)*((px)*(px))))+(((0.0659999999998035)*(pz)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x1;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=0;
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            IKReal x2=((-1.00000000000000)*(py));
            evalcond[0]=x2;
            evalcond[1]=((0.0452399999999216)+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-0.0280000000000200)*(px)))+(((-1.00000000000000)*((px)*(px))))+(((0.0659999999998035)*(pz)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x2;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=((0.500000000000000)*(3.14159265358979));
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            IKReal x3=((0.149999999999636)*(sj3));
            IKReal x4=((x3)+(py));
            evalcond[0]=((-1.00000000000000)*(x4));
            evalcond[1]=((0.0452399999999216)+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-0.0280000000000200)*(px)))+(((-1.00000000000000)*((px)*(px))))+(((0.0659999999998035)*(pz)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x4;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=3.14159265358979;
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            evalcond[0]=((-1.00000000000000)*(py));
            evalcond[1]=((0.0452399999999216)+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-0.0280000000000200)*(px)))+(((-1.00000000000000)*((px)*(px))))+(((0.0659999999998035)*(pz)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=py;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=((-0.500000000000000)*(3.14159265358979));
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
                continue;
            }
            }
            }
            }
            }
            }
            if( numsolutionsj4 == 0 ) { continue; }
            if( j4valid[0] && j4valid[1] && IKabs(cj4array[0]-cj4array[1]) < 0.0001 && IKabs(sj4array[0]-sj4array[1]) < 0.0001 )
            j4valid[1]=false;
        for(int ij4 = 0; ij4 < numsolutionsj4; ++ij4)
        {
        if( !j4valid[ij4] ) { continue; }
        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
        
        {
        IKReal evalcond[12]; int numsolutionsj5 = 0;
        IKReal j5array[1], cj5array[1], sj5array[1];
        bool j5valid[1]={false};
            {
            evalcond[0]=(((((0.0329999999999018)+(((-1.00000000000000)*(pz)))))*(((0.0329999999999018)+(((-1.00000000000000)*(pz))))))+((((0.0140000000000100)+(px)))*(((0.0140000000000100)+(px)))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x5=((0.0329999999999018)+(((-1.00000000000000)*(pz))));
                IKReal x6=((0.0140000000000100)+(px));
                IKReal x7=(x5)*(x5);
                IKReal x8=(x6)*(x6);
                IKReal x9=((x7)+(x8));
                IKReal x10=((IKabs(x9) != 0)?((IKReal)1/(x9)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x10)*(((((0.149999999999636)*(sj3)*(sj4)*(x5)))+(((x6)*(((0.155000000000655)+(((0.149999999999636)*(cj3)))))))))), ((x10)*(((((0.149999999999636)*(sj3)*(sj4)*(x6)))+(((-1.00000000000000)*(x5)*(((0.155000000000655)+(((0.149999999999636)*(cj3)))))))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            evalcond[0]=(((((0.155000000000655)+(((0.149999999999636)*(cj3)))))*(((0.155000000000655)+(((0.149999999999636)*(cj3))))))+(((0.0224999999998090)*((sj3)*(sj3))*((sj4)*(sj4)))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x11=((0.149999999999636)*(cj3));
                IKReal x12=((0.155000000000655)+(x11));
                IKReal x13=(x12)*(x12);
                IKReal x14=(sj3)*(sj3);
                IKReal x15=(sj4)*(sj4);
                IKReal x16=((0.0224999999998090)*(x14)*(x15));
                IKReal x17=((x13)+(x16));
                IKReal x18=((IKabs(x17) != 0)?((IKReal)1/(x17)):(IKReal)1.0e30);
                IKReal x19=((0.0329999999999018)+(((-1.00000000000000)*(pz))));
                IKReal x20=((0.0140000000000100)+(px));
                j5array[0]=IKatan2(((x18)*(((((x12)*(x20)))+(((0.149999999999636)*(sj3)*(sj4)*(x19)))))), ((x18)*(((((0.149999999999636)*(sj3)*(sj4)*(x20)))+(((-1.00000000000000)*(x19)*(((0.155000000000655)+(x11)))))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            evalcond[0]=((((-1.00000000000000)*(((0.155000000000655)+(((0.149999999999636)*(cj3)))))*(((0.0140000000000100)+(px)))))+(((0.149999999999636)*(sj3)*(sj4)*(((0.0329999999999018)+(((-1.00000000000000)*(pz))))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x21=((0.0140000000000100)+(px));
                IKReal x22=((0.149999999999636)*(cj3));
                IKReal x23=((0.155000000000655)+(x22));
                IKReal x24=((0.0329999999999018)+(((-1.00000000000000)*(pz))));
                IKReal x25=((0.149999999999636)*(sj3)*(sj4)*(x24));
                IKReal x26=((x21)*(x23));
                IKReal x27=((x25)+(((-1.00000000000000)*(x26))));
                IKReal x28=((IKabs(x27) != 0)?((IKReal)1/(x27)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x28)*(((((0.0224999999998090)*((sj3)*(sj3))*((sj4)*(sj4))))+(((-1.00000000000000)*((x21)*(x21))))))), ((x28)*(((((x21)*(x24)))+(((-0.149999999999636)*(sj3)*(sj4)*(x23)))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            evalcond[0]=((((0.149999999999636)*(sj3)*(sj4)*(((0.0140000000000100)+(px)))))+(((((0.0329999999999018)+(((-1.00000000000000)*(pz)))))*(((0.155000000000655)+(((0.149999999999636)*(cj3))))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x29=((0.0140000000000100)+(px));
                IKReal x30=((0.0329999999999018)+(((-1.00000000000000)*(pz))));
                IKReal x31=((0.149999999999636)*(cj3));
                IKReal x32=((0.155000000000655)+(x31));
                IKReal x33=((x30)*(x32));
                IKReal x34=((0.149999999999636)*(sj3)*(sj4)*(x29));
                IKReal x35=((x33)+(x34));
                IKReal x36=((IKabs(x35) != 0)?((IKReal)1/(x35)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x36)*(((((x29)*(x30)))+(((0.149999999999636)*(sj3)*(sj4)*(x32)))))), ((x36)*(((((0.0224999999998090)*((sj3)*(sj3))*((sj4)*(sj4))))+(((-1.00000000000000)*((x30)*(x30))))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x37=((0.149999999999636)*(cj3));
            IKReal x38=((0.188000000000102)+(x37));
            IKReal x39=((((-1.00000000000000)*(pz)))+(x38));
            IKReal x40=((0.149999999999636)*(sj3)*(sj4));
            IKReal x41=((-0.0140000000000100)+(x40));
            IKReal x42=((((-1.00000000000000)*(px)))+(x41));
            IKReal x43=(px)*(px);
            IKReal x44=(pz)*(pz);
            IKReal x45=((0.0280000000000200)*(px));
            IKReal x46=(py)*(py);
            IKReal x47=((x46)+(x44)+(x45)+(x43));
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.0452399999999216)+(((-1.00000000000000)*(x47)))+(((0.0659999999998035)*(pz)))+(((0.310000000001310)*(x37))));
            evalcond[2]=x42;
            evalcond[3]=x39;
            evalcond[4]=x39;
            evalcond[5]=x42;
            evalcond[6]=((((-1.00000000000000)*(py)*(sj4)))+(((0.0140000000000100)*(cj4)))+(((35.7142857140861)*(cj4)*(x45))));
            evalcond[7]=((((-1.00000000000000)*(cj4)*(py)))+(((-0.0140000000000100)*(sj4)))+(((-35.7142857140861)*(sj4)*(x45)))+(((0.149999999999636)*(sj3))));
            evalcond[8]=((-0.0130400000003874)+(((-1.00000000000000)*(x47)))+(((0.376000000000204)*(pz))));
            evalcond[9]=((0.0580399999998917)+(((-1.00000000000000)*(x46)))+(((-0.0279999999997926)*(x40)))+(((-1.00000000000000)*(x43)))+(((0.376000000000204)*(x37)))+(((-1.00000000000000)*(x44))));
            evalcond[10]=((((-0.0933333333332484)*(sj4)*(x37)))+(((-0.188000000000102)*(sj3)))+(((pz)*(sj3)))+(((-238.095238095149)*(sj4)*(x37)*(x45)))+(((-6.66666666668607)*(cj4)*(py)*(x37))));
            evalcond[11]=((0.149999999999636)+(((-0.0933333333332484)*(x40)))+(((-6.66666666668607)*(pz)*(x37)))+(((1.25333333334129)*(x37)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((-238.095238095149)*(x40)*(x45))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=0;
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x48=((0.149999999999636)*(cj3));
            IKReal x49=((0.141000000001441)+(x48));
            IKReal x50=((((-1.00000000000000)*(px)))+(x49));
            IKReal x51=((0.149999999999636)*(sj3)*(sj4));
            IKReal x52=((0.0659999999998035)*(pz));
            IKReal x53=(px)*(px);
            IKReal x54=(pz)*(pz);
            IKReal x55=(py)*(py);
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.0452399999999216)+(x52)+(((-1.00000000000000)*(x55)))+(((-1.00000000000000)*(x54)))+(((0.310000000001310)*(x48)))+(((-0.0280000000000200)*(px)))+(((-1.00000000000000)*(x53))));
            evalcond[2]=x50;
            evalcond[3]=((0.0329999999999018)+(((-1.00000000000000)*(pz)))+(((-1.00000000000000)*(x51))));
            evalcond[4]=x50;
            evalcond[5]=((-0.0329999999999018)+(x51)+(pz));
            evalcond[6]=((((-1.00000000000000)*(py)*(sj4)))+(((-15.1515151516069)*(cj4)*(x52)))+(((0.0329999999999018)*(cj4))));
            evalcond[7]=((((15.1515151516069)*(sj4)*(x52)))+(((-1.00000000000000)*(cj4)*(py)))+(((0.149999999999636)*(sj3)))+(((-0.0329999999999018)*(sj4))));
            evalcond[8]=((0.00152999999960457)+(x52)+(((0.282000000002881)*(px)))+(((-1.00000000000000)*(x55)))+(((-1.00000000000000)*(x54)))+(((-1.00000000000000)*(x53))));
            evalcond[9]=((0.0434700000000703)+(((-1.00000000000000)*(x55)))+(((-1.00000000000000)*(x54)))+(((0.282000000002881)*(x48)))+(((-1.00000000000000)*(x53)))+(((-0.0659999999998035)*(x51))));
            evalcond[10]=((((-6.66666666668607)*(cj4)*(py)*(x48)))+(((-0.141000000001441)*(sj3)))+(((101.010101011023)*(sj4)*(x48)*(x52)))+(((-0.219999999999345)*(sj4)*(x48)))+(((px)*(sj3))));
            evalcond[11]=((0.149999999999636)+(((101.010101011023)*(x51)*(x52)))+(((0.940000000009604)*(x48)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((-0.219999999999345)*(x51)))+(((-6.66666666668607)*(px)*(x48))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=((0.500000000000000)*(3.14159265358979));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x56=((0.149999999999636)*(cj3));
            IKReal x57=((0.149999999999636)*(sj3)*(sj4));
            IKReal x58=(px)*(px);
            IKReal x59=(pz)*(pz);
            IKReal x60=((0.0280000000000200)*(px));
            IKReal x61=(py)*(py);
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.0452399999999216)+(((-1.00000000000000)*(x59)))+(((-1.00000000000000)*(x58)))+(((-1.00000000000000)*(x60)))+(((0.310000000001310)*(x56)))+(((0.0659999999998035)*(pz)))+(((-1.00000000000000)*(x61))));
            evalcond[2]=((-0.0140000000000100)+(((-1.00000000000000)*(px)))+(((-1.00000000000000)*(x57))));
            evalcond[3]=((-0.122000000001208)+(((-1.00000000000000)*(pz)))+(((-1.00000000000000)*(x56))));
            evalcond[4]=((0.122000000001208)+(x56)+(pz));
            evalcond[5]=((0.0140000000000100)+(x57)+(px));
            evalcond[6]=((((-1.00000000000000)*(py)*(sj4)))+(((-35.7142857140861)*(cj4)*(x60)))+(((-0.0140000000000100)*(cj4))));
            evalcond[7]=((((0.0140000000000100)*(sj4)))+(((-1.00000000000000)*(cj4)*(py)))+(((0.149999999999636)*(sj3)))+(((35.7142857140861)*(sj4)*(x60))));
            evalcond[8]=((0.00741999999956988)+(((-1.00000000000000)*(x59)))+(((-1.00000000000000)*(x58)))+(((-1.00000000000000)*(x60)))+(((-1.00000000000000)*(x61)))+(((-0.244000000002416)*(pz))));
            evalcond[9]=((0.0375800000001618)+(((-1.00000000000000)*(x59)))+(((0.244000000000597)*(x56)))+(((-1.00000000000000)*(x58)))+(((-1.00000000000000)*(x61)))+(((0.0279999999997926)*(x57))));
            evalcond[10]=((((-1.00000000000000)*(pz)*(sj3)))+(((-0.122000000001208)*(sj3)))+(((-6.66666666668607)*(cj4)*(py)*(x56)))+(((0.0933333333332484)*(sj4)*(x56)))+(((238.095238095149)*(sj4)*(x56)*(x60))));
            evalcond[11]=((0.149999999999636)+(((238.095238095149)*(x57)*(x60)))+(((0.813333333346236)*(x56)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((0.0933333333332484)*(x57)))+(((6.66666666668607)*(pz)*(x56))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=3.14159265358979;
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x62=((0.149999999999636)*(cj3));
            IKReal x63=((0.149999999999636)*(sj3)*(sj4));
            IKReal x64=((0.0329999999999018)+(x63));
            IKReal x65=((((-1.00000000000000)*(pz)))+(x64));
            IKReal x66=((0.0659999999998035)*(pz));
            IKReal x67=(px)*(px);
            IKReal x68=(pz)*(pz);
            IKReal x69=(py)*(py);
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.0452399999999216)+(x66)+(((-1.00000000000000)*(x67)))+(((-0.0280000000000200)*(px)))+(((0.310000000001310)*(x62)))+(((-1.00000000000000)*(x69)))+(((-1.00000000000000)*(x68))));
            evalcond[2]=((-0.168999999999869)+(((-1.00000000000000)*(x62)))+(((-1.00000000000000)*(px))));
            evalcond[3]=x65;
            evalcond[4]=((0.168999999999869)+(x62)+(px));
            evalcond[5]=x65;
            evalcond[6]=((((15.1515151516069)*(cj4)*(x66)))+(((-1.00000000000000)*(py)*(sj4)))+(((-0.0329999999999018)*(cj4))));
            evalcond[7]=((((0.0329999999999018)*(sj4)))+(((-1.00000000000000)*(cj4)*(py)))+(((-15.1515151516069)*(sj4)*(x66)))+(((0.149999999999636)*(sj3))));
            evalcond[8]=((-0.00715000000042210)+(((-0.337999999999738)*(px)))+(x66)+(((-1.00000000000000)*(x67)))+(((-1.00000000000000)*(x69)))+(((-1.00000000000000)*(x68))));
            evalcond[9]=((0.0521499999999833)+(((0.0659999999998035)*(x63)))+(((-1.00000000000000)*(x67)))+(((-1.00000000000000)*(x69)))+(((0.337999999999738)*(x62)))+(((-1.00000000000000)*(x68))));
            evalcond[10]=((((-1.00000000000000)*(px)*(sj3)))+(((-0.168999999999869)*(sj3)))+(((-101.010101011023)*(sj4)*(x62)*(x66)))+(((-6.66666666668607)*(cj4)*(py)*(x62)))+(((0.219999999999345)*(sj4)*(x62))));
            evalcond[11]=((0.149999999999636)+(((1.12666666666337)*(x62)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((6.66666666668607)*(px)*(x62)))+(((0.219999999999345)*(x63)))+(((-101.010101011023)*(x63)*(x66))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=((-0.500000000000000)*(3.14159265358979));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
                continue;
            }
            }
            }
            }
            }
            }
            }
            }
            }
            if( numsolutionsj5 == 0 ) { continue; }
            for(int ij5 = 0; ij5 < numsolutionsj5; ++ij5)
        {
        if( !j5valid[ij5] ) { continue; }
        j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
        
        new_r00=((((-1.00000000000000)*(cj5)*(r20)*(sj3)))+(((cj3)*(cj5)*(r00)*(sj4)))+(((cj3)*(cj4)*(r10)))+(((-1.00000000000000)*(cj3)*(r20)*(sj4)*(sj5)))+(((-1.00000000000000)*(r00)*(sj3)*(sj5))));
        new_r01=((((cj3)*(cj5)*(r01)*(sj4)))+(((-1.00000000000000)*(r01)*(sj3)*(sj5)))+(((cj3)*(cj4)*(r11)))+(((-1.00000000000000)*(cj5)*(r21)*(sj3)))+(((-1.00000000000000)*(cj3)*(r21)*(sj4)*(sj5))));
        new_r02=((((-1.00000000000000)*(cj3)*(r22)*(sj4)*(sj5)))+(((cj3)*(cj4)*(r12)))+(((-1.00000000000000)*(r02)*(sj3)*(sj5)))+(((-1.00000000000000)*(cj5)*(r22)*(sj3)))+(((cj3)*(cj5)*(r02)*(sj4))));
        new_r10=((((cj4)*(r20)*(sj5)))+(((r10)*(sj4)))+(((-1.00000000000000)*(cj4)*(cj5)*(r00))));
        new_r11=((((cj4)*(r21)*(sj5)))+(((-1.00000000000000)*(cj4)*(cj5)*(r01)))+(((r11)*(sj4))));
        new_r12=((((r12)*(sj4)))+(((cj4)*(r22)*(sj5)))+(((-1.00000000000000)*(cj4)*(cj5)*(r02))));
        new_r20=((((cj5)*(r00)*(sj3)*(sj4)))+(((cj3)*(r00)*(sj5)))+(((-1.00000000000000)*(r20)*(sj3)*(sj4)*(sj5)))+(((cj3)*(cj5)*(r20)))+(((cj4)*(r10)*(sj3))));
        new_r21=((((cj4)*(r11)*(sj3)))+(((cj3)*(r01)*(sj5)))+(((-1.00000000000000)*(r21)*(sj3)*(sj4)*(sj5)))+(((cj3)*(cj5)*(r21)))+(((cj5)*(r01)*(sj3)*(sj4))));
        new_r22=((((cj4)*(r12)*(sj3)))+(((-1.00000000000000)*(r22)*(sj3)*(sj4)*(sj5)))+(((cj3)*(r02)*(sj5)))+(((cj3)*(cj5)*(r22)))+(((cj5)*(r02)*(sj3)*(sj4))));
        {
        IKReal j1eval;
        j1eval=new_r21;
        if( j1eval >= 0.999990 && j1eval <= 1.000010 )
        {
            {
            j1 = -1.570796; sj1 = -1.000000; cj1 = 0.000000;
            IKReal j0mul = 1;
            j0=0;
            IKReal j2mul=1.00000000000000;
            j2=((-1.00000000000000)*(IKatan2(new_r10, new_r00)));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[0].fmul = j0mul;
            solution.basesol[0].freeind = 0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[2].fmul = j2mul;
            solution.basesol[2].freeind = 0;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.vfree.resize(1);
            solution.vfree[0] = 0;
            }
            
        } else
        if( j1eval >= -1.000010 && j1eval <= -0.999990 )
        {
            {
            j1 = 1.570796; sj1 = 1.000000; cj1 = 0.000000;
            IKReal j0mul = 1;
            j0=0;
            IKReal j2mul=-1.00000000000000;
            j2=((((-1.00000000000000)*(IKatan2(new_r02, ((-1.00000000000000)*(new_r12))))))+(((-1.00000000000000)*(3.14159265358979))));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[0].fmul = j0mul;
            solution.basesol[0].freeind = 0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[2].fmul = j2mul;
            solution.basesol[2].freeind = 0;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.vfree.resize(1);
            solution.vfree[0] = 0;
            }
            
        } else
        {
            {
            IKReal j0array[2], cj0array[2], sj0array[2];
            bool j0valid[2]={false};
            j0array[0]=IKatan2(((-1.00000000000000)*(new_r20)), new_r22);
            sj0array[0]=IKsin(j0array[0]);
            cj0array[0]=IKcos(j0array[0]);
            j0array[1] = j0array[0] > 0 ? j0array[0]-IKPI : j0array[0]+IKPI;
            sj0array[1] = -sj0array[0];
            cj0array[1] = -cj0array[0];
            if( j0array[0] > IKPI )
                j0array[0]-=IK2PI;
            else if( j0array[0] < -IKPI )
                j0array[0]+=IK2PI;
            j0valid[0] = true;
            if( j0array[1] > IKPI )
                j0array[1]-=IK2PI;
            else if( j0array[1] < -IKPI )
                j0array[1]+=IK2PI;
            j0valid[1] = true;
            if( j0valid[0] && j0valid[1] && IKabs(cj0array[0]-cj0array[1]) < 0.0001 && IKabs(sj0array[0]-sj0array[1]) < 0.0001 )
                j0valid[1]=false;
            for(int ij0 = 0; ij0 < 2; ++ij0)
            {
            if( !j0valid[ij0] ) { continue; }
            j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
            
            {
            IKReal evalcond;
                evalcond=IKcos(j0);
                if( IKabs(evalcond) < 0.00001 )
                {
                {
                IKReal j2array[1], cj2array[1], sj2array[1];
                bool j2valid[1]={false};
                IKReal x70=IKsin(j0);
                IKReal x71=((IKabs(x70) != 0)?((IKReal)1/(x70)):(IKReal)1.0e30);
                j2array[0]=IKatan2(((-1.00000000000000)*(new_r12)*(x71)), ((new_r02)*(x71)));
                sj2array[0]=IKsin(j2array[0]);
                cj2array[0]=IKcos(j2array[0]);
                if( j2array[0] > IKPI )
                    j2array[0]-=IK2PI;
                else if( j2array[0] < -IKPI )
                    j2array[0]+=IK2PI;
                j2valid[0] = true;
                for(int ij2 = 0; ij2 < 1; ++ij2)
                {
                if( !j2valid[ij2] ) { continue; }
                j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                
                {
                IKReal j1array[1], cj1array[1], sj1array[1];
                bool j1valid[1]={false};
                j1array[0]=IKatan2(((-1.00000000000000)*(new_r21)), ((-1.00000000000000)*(new_r20)*(((IKabs(IKsin(j0)) != 0)?((IKReal)1/(IKsin(j0))):(IKReal)1.0e30))));
                sj1array[0]=IKsin(j1array[0]);
                cj1array[0]=IKcos(j1array[0]);
                if( j1array[0] > IKPI )
                    j1array[0]-=IK2PI;
                else if( j1array[0] < -IKPI )
                    j1array[0]+=IK2PI;
                j1valid[0] = true;
                for(int ij1 = 0; ij1 < 1; ++ij1)
                {
                if( !j1valid[ij1] ) { continue; }
                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                
                vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                solution.basesol.resize(7);
                solution.basesol[0].foffset = j0;
                solution.basesol[1].foffset = j1;
                solution.basesol[2].foffset = j2;
                solution.basesol[3].foffset = j3;
                solution.basesol[4].foffset = j4;
                solution.basesol[5].foffset = j5;
                solution.basesol[6].foffset = j6;
                solution.vfree.resize(0);
                }
                }
                }
                }
                
            } else
            {
                if( 1 )
                {
                {
                IKReal j1array[1], cj1array[1], sj1array[1];
                bool j1valid[1]={false};
                j1array[0]=IKatan2(((-1.00000000000000)*(new_r21)), ((new_r22)*(((IKabs(cj0) != 0)?((IKReal)1/(cj0)):(IKReal)1.0e30))));
                sj1array[0]=IKsin(j1array[0]);
                cj1array[0]=IKcos(j1array[0]);
                if( j1array[0] > IKPI )
                    j1array[0]-=IK2PI;
                else if( j1array[0] < -IKPI )
                    j1array[0]+=IK2PI;
                j1valid[0] = true;
                for(int ij1 = 0; ij1 < 1; ++ij1)
                {
                if( !j1valid[ij1] ) { continue; }
                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                
                {
                IKReal evalcond;
                    evalcond=IKcos(j1);
                    if( IKabs(evalcond) < 0.00001 )
                    {
                    {
                    IKReal j2array[1], cj2array[1], sj2array[1];
                    bool j2valid[1]={false};
                    IKReal x72=IKsin(j1);
                    IKReal x73=((IKabs(x72) != 0)?((IKReal)1/(x72)):(IKReal)1.0e30);
                    IKReal x74=IKsin(j0);
                    IKReal x75=IKcos(j0);
                    j2array[0]=IKatan2(((((new_r02)*(x73)*(x75)))+(((-1.00000000000000)*(new_r00)*(x73)*(x74)))), ((((new_r00)*(x75)))+(((new_r02)*(x74)))));
                    sj2array[0]=IKsin(j2array[0]);
                    cj2array[0]=IKcos(j2array[0]);
                    if( j2array[0] > IKPI )
                        j2array[0]-=IK2PI;
                    else if( j2array[0] < -IKPI )
                        j2array[0]+=IK2PI;
                    j2valid[0] = true;
                    for(int ij2 = 0; ij2 < 1; ++ij2)
                    {
                    if( !j2valid[ij2] ) { continue; }
                    j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                    
                    vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                    solution.basesol.resize(7);
                    solution.basesol[0].foffset = j0;
                    solution.basesol[1].foffset = j1;
                    solution.basesol[2].foffset = j2;
                    solution.basesol[3].foffset = j3;
                    solution.basesol[4].foffset = j4;
                    solution.basesol[5].foffset = j5;
                    solution.basesol[6].foffset = j6;
                    solution.vfree.resize(0);
                    }
                    }
                    
                } else
                {
                    if( 1 )
                    {
                    {
                    IKReal j2array[1], cj2array[1], sj2array[1];
                    bool j2valid[1]={false};
                    IKReal x76=IKcos(j1);
                    IKReal x77=((IKabs(x76) != 0)?((IKReal)1/(x76)):(IKReal)1.0e30);
                    j2array[0]=IKatan2(((new_r01)*(x77)), ((new_r11)*(x77)));
                    sj2array[0]=IKsin(j2array[0]);
                    cj2array[0]=IKcos(j2array[0]);
                    if( j2array[0] > IKPI )
                        j2array[0]-=IK2PI;
                    else if( j2array[0] < -IKPI )
                        j2array[0]+=IK2PI;
                    j2valid[0] = true;
                    for(int ij2 = 0; ij2 < 1; ++ij2)
                    {
                    if( !j2valid[ij2] ) { continue; }
                    j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                    
                    vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                    solution.basesol.resize(7);
                    solution.basesol[0].foffset = j0;
                    solution.basesol[1].foffset = j1;
                    solution.basesol[2].foffset = j2;
                    solution.basesol[3].foffset = j3;
                    solution.basesol[4].foffset = j4;
                    solution.basesol[5].foffset = j5;
                    solution.basesol[6].foffset = j6;
                    solution.vfree.resize(0);
                    }
                    }
                    
                } else
                {
                }
                }
                }
                }
                }
                
            } else
            {
            }
            }
            }
            }
            }
            
        }
        }
        }
        }
        }
        }
        }
        }
        
    } else
    {
        evalcond=((((-0.500000000000000)*(3.14159265358979)))+(j6));
        if( IKabs(evalcond) < 0.00001 )
        {
        {
        IKReal j3array[2], cj3array[2], sj3array[2];
        bool j3valid[2]={false};
        cj3array[0]=((-0.972903225803748)+(((1.41935483871202)*(px)))+(((21.5053763440810)*((py)*(py))))+(((21.5053763440810)*((pz)*(pz))))+(((0.602150537633861)*(pz)))+(((21.5053763440810)*((px)*(px)))));
        if( cj3array[0] >= -1.0001 && cj3array[0] <= 1.0001 )
        {
            j3valid[0] = j3valid[1] = true;
            j3array[0] = IKacos(cj3array[0]);
            sj3array[0] = IKsin(j3array[0]);
            cj3array[1] = cj3array[0];
            j3array[1] = -j3array[0];
            sj3array[1] = -sj3array[0];
        }
        else if( isnan(cj3array[0]) )
        {
            // probably any value will work
            j3valid[0] = true;
            cj3array[0] = 1; sj3array[0] = 0; j3array[0] = 0;
        }
        if( j3valid[0] && j3valid[1] && IKabs(cj3array[0]-cj3array[1]) < 0.0001 && IKabs(sj3array[0]-sj3array[1]) < 0.0001 )
            j3valid[1]=false;
        for(int ij3 = 0; ij3 < 2; ++ij3)
        {
        if( !j3valid[ij3] ) { continue; }
        j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
        
        {
        IKReal evalcond[3]; int numsolutionsj4 = 0;
        IKReal j4array[2], cj4array[2], sj4array[2];
        bool j4valid[2]={false};
            {
            evalcond[0]=sj3;
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                cj4array[0]=((6.66666666668607)*(py)*(((IKabs(sj3) != 0)?((IKReal)1/(sj3)):(IKReal)1.0e30)));
                if( cj4array[0] >= -1.0001 && cj4array[0] <= 1.0001 )
                {
                    j4valid[0] = j4valid[1] = true;
                    j4array[0] = IKacos(cj4array[0]);
                    sj4array[0] = IKsin(j4array[0]);
                    cj4array[1] = cj4array[0];
                    j4array[1] = -j4array[0];
                    sj4array[1] = -sj4array[0];
                }
                else if( isnan(cj4array[0]) )
                {
                    // probably any value will work
                    j4valid[0] = true;
                    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
                }
                numsolutionsj4 = 2;
                
            } else
            {
            IKReal x78=((0.149999999999636)*(sj3));
            IKReal x79=((((-1.00000000000000)*(py)))+(x78));
            evalcond[0]=x79;
            evalcond[1]=((0.0452399999999216)+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-0.0659999999998035)*(px)))+(((-1.00000000000000)*((px)*(px))))+(((-0.0280000000000200)*(pz)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x79;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=0;
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            IKReal x80=((-1.00000000000000)*(py));
            evalcond[0]=x80;
            evalcond[1]=((0.0452399999999216)+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-0.0659999999998035)*(px)))+(((-1.00000000000000)*((px)*(px))))+(((-0.0280000000000200)*(pz)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x80;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=((0.500000000000000)*(3.14159265358979));
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            IKReal x81=((0.149999999999636)*(sj3));
            IKReal x82=((py)+(x81));
            evalcond[0]=((-1.00000000000000)*(x82));
            evalcond[1]=((0.0452399999999216)+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-0.0659999999998035)*(px)))+(((-1.00000000000000)*((px)*(px))))+(((-0.0280000000000200)*(pz)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x82;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=3.14159265358979;
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            evalcond[0]=((-1.00000000000000)*(py));
            evalcond[1]=((0.0452399999999216)+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-0.0659999999998035)*(px)))+(((-1.00000000000000)*((px)*(px))))+(((-0.0280000000000200)*(pz)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=py;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=((-0.500000000000000)*(3.14159265358979));
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
                continue;
            }
            }
            }
            }
            }
            }
            if( numsolutionsj4 == 0 ) { continue; }
            if( j4valid[0] && j4valid[1] && IKabs(cj4array[0]-cj4array[1]) < 0.0001 && IKabs(sj4array[0]-sj4array[1]) < 0.0001 )
            j4valid[1]=false;
        for(int ij4 = 0; ij4 < numsolutionsj4; ++ij4)
        {
        if( !j4valid[ij4] ) { continue; }
        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
        
        {
        IKReal evalcond[12]; int numsolutionsj5 = 0;
        IKReal j5array[1], cj5array[1], sj5array[1];
        bool j5valid[1]={false};
            {
            evalcond[0]=(((((0.0329999999999018)+(px)))*(((0.0329999999999018)+(px))))+((((0.0140000000000100)+(pz)))*(((0.0140000000000100)+(pz)))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x83=((0.0140000000000100)+(pz));
                IKReal x84=((0.0329999999999018)+(px));
                IKReal x85=(x84)*(x84);
                IKReal x86=(x83)*(x83);
                IKReal x87=((x86)+(x85));
                IKReal x88=((IKabs(x87) != 0)?((IKReal)1/(x87)):(IKReal)1.0e30);
                IKReal x89=((0.149999999999636)*(cj3));
                IKReal x90=((0.155000000000655)+(x89));
                j5array[0]=IKatan2(((x88)*(((((0.149999999999636)*(sj3)*(sj4)*(x84)))+(((x83)*(x90)))))), ((x88)*(((((0.149999999999636)*(sj3)*(sj4)*(x83)))+(((-1.00000000000000)*(x84)*(x90)))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            evalcond[0]=((((((0.155000000000655)+(((0.149999999999636)*(cj3)))))*(((0.0140000000000100)+(pz)))))+(((-0.149999999999636)*(sj3)*(sj4)*(((0.0329999999999018)+(px))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x91=((0.149999999999636)*(cj3));
                IKReal x92=((0.155000000000655)+(x91));
                IKReal x93=((0.0329999999999018)+(px));
                IKReal x94=((0.0140000000000100)+(pz));
                IKReal x95=((x92)*(x94));
                IKReal x96=((0.149999999999636)*(sj3)*(sj4)*(x93));
                IKReal x97=((((-1.00000000000000)*(x96)))+(x95));
                IKReal x98=((IKabs(x97) != 0)?((IKReal)1/(x97)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x98)*((((x92)*(x92))+(((-1.00000000000000)*((x93)*(x93))))))), ((x98)*(((((-1.00000000000000)*(x93)*(x94)))+(((0.149999999999636)*(sj3)*(sj4)*(x92)))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            evalcond[0]=(((((0.155000000000655)+(((0.149999999999636)*(cj3)))))*(((0.155000000000655)+(((0.149999999999636)*(cj3))))))+(((0.0224999999998090)*((sj3)*(sj3))*((sj4)*(sj4)))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x99=((0.149999999999636)*(cj3));
                IKReal x100=((0.155000000000655)+(x99));
                IKReal x101=(x100)*(x100);
                IKReal x102=(sj3)*(sj3);
                IKReal x103=(sj4)*(sj4);
                IKReal x104=((0.0224999999998090)*(x102)*(x103));
                IKReal x105=((x104)+(x101));
                IKReal x106=((IKabs(x105) != 0)?((IKReal)1/(x105)):(IKReal)1.0e30);
                IKReal x107=((0.0140000000000100)+(pz));
                IKReal x108=((0.0329999999999018)+(px));
                j5array[0]=IKatan2(((x106)*(((((x100)*(x107)))+(((0.149999999999636)*(sj3)*(sj4)*(x108)))))), ((x106)*(((((-1.00000000000000)*(x100)*(x108)))+(((0.149999999999636)*(sj3)*(sj4)*(x107)))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            evalcond[0]=((((-1.00000000000000)*(((0.0329999999999018)+(px)))*(((0.155000000000655)+(((0.149999999999636)*(cj3)))))))+(((-0.149999999999636)*(sj3)*(sj4)*(((0.0140000000000100)+(pz))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x109=((0.0329999999999018)+(px));
                IKReal x110=((0.149999999999636)*(cj3));
                IKReal x111=((0.155000000000655)+(x110));
                IKReal x112=((0.0140000000000100)+(pz));
                IKReal x113=((x109)*(x111));
                IKReal x114=((0.149999999999636)*(sj3)*(sj4)*(x112));
                IKReal x115=((x113)+(x114));
                IKReal x116=((-1.00000000000000)*(x115));
                IKReal x117=((IKabs(x116) != 0)?((IKReal)1/(x116)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x117)*(((((-0.149999999999636)*(sj3)*(sj4)*(x111)))+(((-1.00000000000000)*(x109)*(x112)))))), ((x117)*(((((-1.00000000000000)*((x112)*(x112))))+((x111)*(x111))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x118=((0.149999999999636)*(cj3));
            IKReal x119=((0.149999999999636)*(sj3)*(sj4));
            IKReal x120=((-0.0140000000000100)+(x119));
            IKReal x121=((((-1.00000000000000)*(pz)))+(x120));
            IKReal x122=(px)*(px);
            IKReal x123=(pz)*(pz);
            IKReal x124=((0.0280000000000200)*(pz));
            IKReal x125=(py)*(py);
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.0452399999999216)+(((-1.00000000000000)*(x124)))+(((0.310000000001310)*(x118)))+(((-1.00000000000000)*(x122)))+(((-0.0659999999998035)*(px)))+(((-1.00000000000000)*(x125)))+(((-1.00000000000000)*(x123))));
            evalcond[2]=((0.188000000000102)+(px)+(x118));
            evalcond[3]=((-0.188000000000102)+(((-1.00000000000000)*(x118)))+(((-1.00000000000000)*(px))));
            evalcond[4]=x121;
            evalcond[5]=x121;
            evalcond[6]=((((35.7142857140861)*(cj4)*(x124)))+(((-1.00000000000000)*(py)*(sj4)))+(((0.0140000000000100)*(cj4))));
            evalcond[7]=((-0.0130400000003874)+(((-1.00000000000000)*(x124)))+(((-1.00000000000000)*(x122)))+(((-1.00000000000000)*(x125)))+(((-1.00000000000000)*(x123)))+(((-0.376000000000204)*(px))));
            evalcond[8]=((((-1.00000000000000)*(cj4)*(py)))+(((-0.0140000000000100)*(sj4)))+(((0.149999999999636)*(sj3)))+(((-35.7142857140861)*(sj4)*(x124))));
            evalcond[9]=((0.0580399999998917)+(((-0.0279999999997926)*(x119)))+(((-1.00000000000000)*(x122)))+(((-1.00000000000000)*(x125)))+(((0.376000000000204)*(x118)))+(((-1.00000000000000)*(x123))));
            evalcond[10]=((((-1.00000000000000)*(px)*(sj3)))+(((-0.0933333333332484)*(sj4)*(x118)))+(((-6.66666666668607)*(cj4)*(py)*(x118)))+(((-0.188000000000102)*(sj3)))+(((-238.095238095149)*(sj4)*(x118)*(x124))));
            evalcond[11]=((0.149999999999636)+(((6.66666666668607)*(px)*(x118)))+(((-0.0933333333332484)*(x119)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((-238.095238095149)*(x119)*(x124)))+(((1.25333333334129)*(x118))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=0;
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x126=((0.149999999999636)*(cj3));
            IKReal x127=((0.141000000001441)+(x126));
            IKReal x128=((((-1.00000000000000)*(pz)))+(x127));
            IKReal x129=((0.149999999999636)*(sj3)*(sj4));
            IKReal x130=((-0.0329999999999018)+(x129));
            IKReal x131=((((-1.00000000000000)*(px)))+(x130));
            IKReal x132=(px)*(px);
            IKReal x133=((0.0659999999998035)*(px));
            IKReal x134=(pz)*(pz);
            IKReal x135=(py)*(py);
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.0452399999999216)+(((-1.00000000000000)*(x135)))+(((0.310000000001310)*(x126)))+(((-1.00000000000000)*(x134)))+(((-1.00000000000000)*(x132)))+(((-0.0280000000000200)*(pz)))+(((-1.00000000000000)*(x133))));
            evalcond[2]=x128;
            evalcond[3]=x131;
            evalcond[4]=x128;
            evalcond[5]=x131;
            evalcond[6]=((((15.1515151516069)*(cj4)*(x133)))+(((-1.00000000000000)*(py)*(sj4)))+(((0.0329999999999018)*(cj4))));
            evalcond[7]=((0.00152999999960457)+(((-1.00000000000000)*(x135)))+(((-1.00000000000000)*(x134)))+(((-1.00000000000000)*(x132)))+(((0.282000000002881)*(pz)))+(((-1.00000000000000)*(x133))));
            evalcond[8]=((((-15.1515151516069)*(sj4)*(x133)))+(((-1.00000000000000)*(cj4)*(py)))+(((0.149999999999636)*(sj3)))+(((-0.0329999999999018)*(sj4))));
            evalcond[9]=((0.0434700000000703)+(((-0.0659999999998035)*(x129)))+(((0.282000000002881)*(x126)))+(((-1.00000000000000)*(x135)))+(((-1.00000000000000)*(x134)))+(((-1.00000000000000)*(x132))));
            evalcond[10]=((((-0.219999999999345)*(sj4)*(x126)))+(((-6.66666666668607)*(cj4)*(py)*(x126)))+(((-0.141000000001441)*(sj3)))+(((pz)*(sj3)))+(((-101.010101011023)*(sj4)*(x126)*(x133))));
            evalcond[11]=((0.149999999999636)+(((-101.010101011023)*(x129)*(x133)))+(((0.940000000009604)*(x126)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((-6.66666666668607)*(pz)*(x126)))+(((-0.219999999999345)*(x129))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=((0.500000000000000)*(3.14159265358979));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x136=((0.149999999999636)*(cj3));
            IKReal x137=((0.122000000001208)+(x136));
            IKReal x138=((((-1.00000000000000)*(px)))+(x137));
            IKReal x139=((0.149999999999636)*(sj3)*(sj4));
            IKReal x140=(px)*(px);
            IKReal x141=(pz)*(pz);
            IKReal x142=((0.0280000000000200)*(pz));
            IKReal x143=(py)*(py);
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.0452399999999216)+(((-1.00000000000000)*(x141)))+(((-1.00000000000000)*(x142)))+(((0.310000000001310)*(x136)))+(((-0.0659999999998035)*(px)))+(((-1.00000000000000)*(x140)))+(((-1.00000000000000)*(x143))));
            evalcond[2]=x138;
            evalcond[3]=x138;
            evalcond[4]=((-0.0140000000000100)+(((-1.00000000000000)*(pz)))+(((-1.00000000000000)*(x139))));
            evalcond[5]=((0.0140000000000100)+(pz)+(x139));
            evalcond[6]=((((-1.00000000000000)*(py)*(sj4)))+(((-35.7142857140861)*(cj4)*(x142)))+(((-0.0140000000000100)*(cj4))));
            evalcond[7]=((0.00741999999956988)+(((-1.00000000000000)*(x141)))+(((-1.00000000000000)*(x142)))+(((0.244000000002416)*(px)))+(((-1.00000000000000)*(x140)))+(((-1.00000000000000)*(x143))));
            evalcond[8]=((((35.7142857140861)*(sj4)*(x142)))+(((0.0140000000000100)*(sj4)))+(((-1.00000000000000)*(cj4)*(py)))+(((0.149999999999636)*(sj3))));
            evalcond[9]=((0.0375800000001618)+(((0.0279999999997926)*(x139)))+(((-1.00000000000000)*(x141)))+(((0.244000000000597)*(x136)))+(((-1.00000000000000)*(x140)))+(((-1.00000000000000)*(x143))));
            evalcond[10]=((((-0.122000000001208)*(sj3)))+(((-6.66666666668607)*(cj4)*(py)*(x136)))+(((0.0933333333332484)*(sj4)*(x136)))+(((px)*(sj3)))+(((238.095238095149)*(sj4)*(x136)*(x142))));
            evalcond[11]=((0.149999999999636)+(((238.095238095149)*(x139)*(x142)))+(((0.0933333333332484)*(x139)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((-6.66666666668607)*(px)*(x136)))+(((0.813333333346236)*(x136))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=3.14159265358979;
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x144=((0.149999999999636)*(cj3));
            IKReal x145=((0.149999999999636)*(sj3)*(sj4));
            IKReal x146=(px)*(px);
            IKReal x147=((0.0659999999998035)*(px));
            IKReal x148=(pz)*(pz);
            IKReal x149=(py)*(py);
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.0452399999999216)+(((-1.00000000000000)*(x148)))+(((0.310000000001310)*(x144)))+(((-1.00000000000000)*(x146)))+(((-1.00000000000000)*(x147)))+(((-1.00000000000000)*(x149)))+(((-0.0280000000000200)*(pz))));
            evalcond[2]=((0.168999999999869)+(pz)+(x144));
            evalcond[3]=((-0.0329999999999018)+(((-1.00000000000000)*(px)))+(((-1.00000000000000)*(x145))));
            evalcond[4]=((-0.168999999999869)+(((-1.00000000000000)*(pz)))+(((-1.00000000000000)*(x144))));
            evalcond[5]=((0.0329999999999018)+(px)+(x145));
            evalcond[6]=((((-1.00000000000000)*(py)*(sj4)))+(((-0.0329999999999018)*(cj4)))+(((-15.1515151516069)*(cj4)*(x147))));
            evalcond[7]=((-0.00715000000042210)+(((-1.00000000000000)*(x148)))+(((-0.337999999999738)*(pz)))+(((-1.00000000000000)*(x146)))+(((-1.00000000000000)*(x147)))+(((-1.00000000000000)*(x149))));
            evalcond[8]=((((0.0329999999999018)*(sj4)))+(((15.1515151516069)*(sj4)*(x147)))+(((-1.00000000000000)*(cj4)*(py)))+(((0.149999999999636)*(sj3))));
            evalcond[9]=((0.0521499999999833)+(((-1.00000000000000)*(x148)))+(((-1.00000000000000)*(x146)))+(((0.0659999999998035)*(x145)))+(((0.337999999999738)*(x144)))+(((-1.00000000000000)*(x149))));
            evalcond[10]=((((-1.00000000000000)*(pz)*(sj3)))+(((-6.66666666668607)*(cj4)*(py)*(x144)))+(((0.219999999999345)*(sj4)*(x144)))+(((-0.168999999999869)*(sj3)))+(((101.010101011023)*(sj4)*(x144)*(x147))));
            evalcond[11]=((0.149999999999636)+(((0.219999999999345)*(x145)))+(((101.010101011023)*(x145)*(x147)))+(((6.66666666668607)*(pz)*(x144)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((1.12666666666337)*(x144))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=((-0.500000000000000)*(3.14159265358979));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
                continue;
            }
            }
            }
            }
            }
            }
            }
            }
            }
            if( numsolutionsj5 == 0 ) { continue; }
            for(int ij5 = 0; ij5 < numsolutionsj5; ++ij5)
        {
        if( !j5valid[ij5] ) { continue; }
        j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
        
        new_r00=((((cj3)*(cj5)*(r20)*(sj4)))+(((cj5)*(r00)*(sj3)))+(((cj3)*(r00)*(sj4)*(sj5)))+(((cj3)*(cj4)*(r10)))+(((-1.00000000000000)*(r20)*(sj3)*(sj5))));
        new_r01=((((cj3)*(r01)*(sj4)*(sj5)))+(((cj5)*(r01)*(sj3)))+(((cj3)*(cj4)*(r11)))+(((-1.00000000000000)*(r21)*(sj3)*(sj5)))+(((cj3)*(cj5)*(r21)*(sj4))));
        new_r02=((((cj3)*(r02)*(sj4)*(sj5)))+(((cj5)*(r02)*(sj3)))+(((cj3)*(cj4)*(r12)))+(((-1.00000000000000)*(r22)*(sj3)*(sj5)))+(((cj3)*(cj5)*(r22)*(sj4))));
        new_r10=((((-1.00000000000000)*(cj4)*(cj5)*(r20)))+(((r10)*(sj4)))+(((-1.00000000000000)*(cj4)*(r00)*(sj5))));
        new_r11=((((-1.00000000000000)*(cj4)*(cj5)*(r21)))+(((-1.00000000000000)*(cj4)*(r01)*(sj5)))+(((r11)*(sj4))));
        new_r12=((((r12)*(sj4)))+(((-1.00000000000000)*(cj4)*(cj5)*(r22)))+(((-1.00000000000000)*(cj4)*(r02)*(sj5))));
        new_r20=((((cj3)*(r20)*(sj5)))+(((r00)*(sj3)*(sj4)*(sj5)))+(((cj5)*(r20)*(sj3)*(sj4)))+(((cj4)*(r10)*(sj3)))+(((-1.00000000000000)*(cj3)*(cj5)*(r00))));
        new_r21=((((cj4)*(r11)*(sj3)))+(((-1.00000000000000)*(cj3)*(cj5)*(r01)))+(((r01)*(sj3)*(sj4)*(sj5)))+(((cj3)*(r21)*(sj5)))+(((cj5)*(r21)*(sj3)*(sj4))));
        new_r22=((((cj4)*(r12)*(sj3)))+(((-1.00000000000000)*(cj3)*(cj5)*(r02)))+(((r02)*(sj3)*(sj4)*(sj5)))+(((cj5)*(r22)*(sj3)*(sj4)))+(((cj3)*(r22)*(sj5))));
        {
        IKReal j1eval;
        j1eval=new_r21;
        if( j1eval >= 0.999990 && j1eval <= 1.000010 )
        {
            {
            j1 = -1.570796; sj1 = -1.000000; cj1 = 0.000000;
            IKReal j0mul = 1;
            j0=0;
            IKReal j2mul=1.00000000000000;
            j2=((-1.00000000000000)*(IKatan2(new_r10, new_r00)));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[0].fmul = j0mul;
            solution.basesol[0].freeind = 0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[2].fmul = j2mul;
            solution.basesol[2].freeind = 0;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.vfree.resize(1);
            solution.vfree[0] = 0;
            }
            
        } else
        if( j1eval >= -1.000010 && j1eval <= -0.999990 )
        {
            {
            j1 = 1.570796; sj1 = 1.000000; cj1 = 0.000000;
            IKReal j0mul = 1;
            j0=0;
            IKReal j2mul=-1.00000000000000;
            j2=((((-1.00000000000000)*(IKatan2(new_r02, ((-1.00000000000000)*(new_r12))))))+(((-1.00000000000000)*(3.14159265358979))));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[0].fmul = j0mul;
            solution.basesol[0].freeind = 0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[2].fmul = j2mul;
            solution.basesol[2].freeind = 0;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.vfree.resize(1);
            solution.vfree[0] = 0;
            }
            
        } else
        {
            {
            IKReal j0array[2], cj0array[2], sj0array[2];
            bool j0valid[2]={false};
            j0array[0]=IKatan2(((-1.00000000000000)*(new_r20)), new_r22);
            sj0array[0]=IKsin(j0array[0]);
            cj0array[0]=IKcos(j0array[0]);
            j0array[1] = j0array[0] > 0 ? j0array[0]-IKPI : j0array[0]+IKPI;
            sj0array[1] = -sj0array[0];
            cj0array[1] = -cj0array[0];
            if( j0array[0] > IKPI )
                j0array[0]-=IK2PI;
            else if( j0array[0] < -IKPI )
                j0array[0]+=IK2PI;
            j0valid[0] = true;
            if( j0array[1] > IKPI )
                j0array[1]-=IK2PI;
            else if( j0array[1] < -IKPI )
                j0array[1]+=IK2PI;
            j0valid[1] = true;
            if( j0valid[0] && j0valid[1] && IKabs(cj0array[0]-cj0array[1]) < 0.0001 && IKabs(sj0array[0]-sj0array[1]) < 0.0001 )
                j0valid[1]=false;
            for(int ij0 = 0; ij0 < 2; ++ij0)
            {
            if( !j0valid[ij0] ) { continue; }
            j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
            
            {
            IKReal evalcond;
                evalcond=IKcos(j0);
                if( IKabs(evalcond) < 0.00001 )
                {
                {
                IKReal j2array[1], cj2array[1], sj2array[1];
                bool j2valid[1]={false};
                IKReal x150=IKsin(j0);
                IKReal x151=((IKabs(x150) != 0)?((IKReal)1/(x150)):(IKReal)1.0e30);
                j2array[0]=IKatan2(((-1.00000000000000)*(new_r12)*(x151)), ((new_r02)*(x151)));
                sj2array[0]=IKsin(j2array[0]);
                cj2array[0]=IKcos(j2array[0]);
                if( j2array[0] > IKPI )
                    j2array[0]-=IK2PI;
                else if( j2array[0] < -IKPI )
                    j2array[0]+=IK2PI;
                j2valid[0] = true;
                for(int ij2 = 0; ij2 < 1; ++ij2)
                {
                if( !j2valid[ij2] ) { continue; }
                j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                
                {
                IKReal j1array[1], cj1array[1], sj1array[1];
                bool j1valid[1]={false};
                j1array[0]=IKatan2(((-1.00000000000000)*(new_r21)), ((-1.00000000000000)*(new_r20)*(((IKabs(IKsin(j0)) != 0)?((IKReal)1/(IKsin(j0))):(IKReal)1.0e30))));
                sj1array[0]=IKsin(j1array[0]);
                cj1array[0]=IKcos(j1array[0]);
                if( j1array[0] > IKPI )
                    j1array[0]-=IK2PI;
                else if( j1array[0] < -IKPI )
                    j1array[0]+=IK2PI;
                j1valid[0] = true;
                for(int ij1 = 0; ij1 < 1; ++ij1)
                {
                if( !j1valid[ij1] ) { continue; }
                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                
                vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                solution.basesol.resize(7);
                solution.basesol[0].foffset = j0;
                solution.basesol[1].foffset = j1;
                solution.basesol[2].foffset = j2;
                solution.basesol[3].foffset = j3;
                solution.basesol[4].foffset = j4;
                solution.basesol[5].foffset = j5;
                solution.basesol[6].foffset = j6;
                solution.vfree.resize(0);
                }
                }
                }
                }
                
            } else
            {
                if( 1 )
                {
                {
                IKReal j1array[1], cj1array[1], sj1array[1];
                bool j1valid[1]={false};
                j1array[0]=IKatan2(((-1.00000000000000)*(new_r21)), ((new_r22)*(((IKabs(cj0) != 0)?((IKReal)1/(cj0)):(IKReal)1.0e30))));
                sj1array[0]=IKsin(j1array[0]);
                cj1array[0]=IKcos(j1array[0]);
                if( j1array[0] > IKPI )
                    j1array[0]-=IK2PI;
                else if( j1array[0] < -IKPI )
                    j1array[0]+=IK2PI;
                j1valid[0] = true;
                for(int ij1 = 0; ij1 < 1; ++ij1)
                {
                if( !j1valid[ij1] ) { continue; }
                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                
                {
                IKReal evalcond;
                    evalcond=IKcos(j1);
                    if( IKabs(evalcond) < 0.00001 )
                    {
                    {
                    IKReal j2array[1], cj2array[1], sj2array[1];
                    bool j2valid[1]={false};
                    IKReal x152=IKsin(j1);
                    IKReal x153=((IKabs(x152) != 0)?((IKReal)1/(x152)):(IKReal)1.0e30);
                    IKReal x154=IKsin(j0);
                    IKReal x155=IKcos(j0);
                    j2array[0]=IKatan2(((((-1.00000000000000)*(new_r00)*(x153)*(x154)))+(((new_r02)*(x153)*(x155)))), ((((new_r02)*(x154)))+(((new_r00)*(x155)))));
                    sj2array[0]=IKsin(j2array[0]);
                    cj2array[0]=IKcos(j2array[0]);
                    if( j2array[0] > IKPI )
                        j2array[0]-=IK2PI;
                    else if( j2array[0] < -IKPI )
                        j2array[0]+=IK2PI;
                    j2valid[0] = true;
                    for(int ij2 = 0; ij2 < 1; ++ij2)
                    {
                    if( !j2valid[ij2] ) { continue; }
                    j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                    
                    vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                    solution.basesol.resize(7);
                    solution.basesol[0].foffset = j0;
                    solution.basesol[1].foffset = j1;
                    solution.basesol[2].foffset = j2;
                    solution.basesol[3].foffset = j3;
                    solution.basesol[4].foffset = j4;
                    solution.basesol[5].foffset = j5;
                    solution.basesol[6].foffset = j6;
                    solution.vfree.resize(0);
                    }
                    }
                    
                } else
                {
                    if( 1 )
                    {
                    {
                    IKReal j2array[1], cj2array[1], sj2array[1];
                    bool j2valid[1]={false};
                    IKReal x156=IKcos(j1);
                    IKReal x157=((IKabs(x156) != 0)?((IKReal)1/(x156)):(IKReal)1.0e30);
                    j2array[0]=IKatan2(((new_r01)*(x157)), ((new_r11)*(x157)));
                    sj2array[0]=IKsin(j2array[0]);
                    cj2array[0]=IKcos(j2array[0]);
                    if( j2array[0] > IKPI )
                        j2array[0]-=IK2PI;
                    else if( j2array[0] < -IKPI )
                        j2array[0]+=IK2PI;
                    j2valid[0] = true;
                    for(int ij2 = 0; ij2 < 1; ++ij2)
                    {
                    if( !j2valid[ij2] ) { continue; }
                    j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                    
                    vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                    solution.basesol.resize(7);
                    solution.basesol[0].foffset = j0;
                    solution.basesol[1].foffset = j1;
                    solution.basesol[2].foffset = j2;
                    solution.basesol[3].foffset = j3;
                    solution.basesol[4].foffset = j4;
                    solution.basesol[5].foffset = j5;
                    solution.basesol[6].foffset = j6;
                    solution.vfree.resize(0);
                    }
                    }
                    
                } else
                {
                }
                }
                }
                }
                }
                
            } else
            {
            }
            }
            }
            }
            }
            
        }
        }
        }
        }
        }
        }
        }
        }
        
    } else
    {
        evalcond=((j6)+(((-1.00000000000000)*(3.14159265358979))));
        if( IKabs(evalcond) < 0.00001 )
        {
        {
        IKReal j3array[2], cj3array[2], sj3array[2];
        bool j3valid[2]={false};
        cj3array[0]=((-0.972903225803748)+(((21.5053763440810)*((py)*(py))))+(((21.5053763440810)*((pz)*(pz))))+(((1.41935483871202)*(pz)))+(((-0.602150537633861)*(px)))+(((21.5053763440810)*((px)*(px)))));
        if( cj3array[0] >= -1.0001 && cj3array[0] <= 1.0001 )
        {
            j3valid[0] = j3valid[1] = true;
            j3array[0] = IKacos(cj3array[0]);
            sj3array[0] = IKsin(j3array[0]);
            cj3array[1] = cj3array[0];
            j3array[1] = -j3array[0];
            sj3array[1] = -sj3array[0];
        }
        else if( isnan(cj3array[0]) )
        {
            // probably any value will work
            j3valid[0] = true;
            cj3array[0] = 1; sj3array[0] = 0; j3array[0] = 0;
        }
        if( j3valid[0] && j3valid[1] && IKabs(cj3array[0]-cj3array[1]) < 0.0001 && IKabs(sj3array[0]-sj3array[1]) < 0.0001 )
            j3valid[1]=false;
        for(int ij3 = 0; ij3 < 2; ++ij3)
        {
        if( !j3valid[ij3] ) { continue; }
        j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
        
        {
        IKReal evalcond[3]; int numsolutionsj4 = 0;
        IKReal j4array[2], cj4array[2], sj4array[2];
        bool j4valid[2]={false};
            {
            evalcond[0]=sj3;
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                cj4array[0]=((6.66666666668607)*(py)*(((IKabs(sj3) != 0)?((IKReal)1/(sj3)):(IKReal)1.0e30)));
                if( cj4array[0] >= -1.0001 && cj4array[0] <= 1.0001 )
                {
                    j4valid[0] = j4valid[1] = true;
                    j4array[0] = IKacos(cj4array[0]);
                    sj4array[0] = IKsin(j4array[0]);
                    cj4array[1] = cj4array[0];
                    j4array[1] = -j4array[0];
                    sj4array[1] = -sj4array[0];
                }
                else if( isnan(cj4array[0]) )
                {
                    // probably any value will work
                    j4valid[0] = true;
                    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
                }
                numsolutionsj4 = 2;
                
            } else
            {
            IKReal x158=((0.149999999999636)*(sj3));
            IKReal x159=((((-1.00000000000000)*(py)))+(x158));
            evalcond[0]=x159;
            evalcond[1]=((0.0452399999999216)+(((-0.0659999999998035)*(pz)))+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((px)*(px))))+(((0.0280000000000200)*(px)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x159;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=0;
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            IKReal x160=((-1.00000000000000)*(py));
            evalcond[0]=x160;
            evalcond[1]=((0.0452399999999216)+(((-0.0659999999998035)*(pz)))+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((px)*(px))))+(((0.0280000000000200)*(px)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x160;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=((0.500000000000000)*(3.14159265358979));
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            IKReal x161=((0.149999999999636)*(sj3));
            IKReal x162=((x161)+(py));
            evalcond[0]=((-1.00000000000000)*(x162));
            evalcond[1]=((0.0452399999999216)+(((-0.0659999999998035)*(pz)))+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((px)*(px))))+(((0.0280000000000200)*(px)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x162;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=3.14159265358979;
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            evalcond[0]=((-1.00000000000000)*(py));
            evalcond[1]=((0.0452399999999216)+(((-0.0659999999998035)*(pz)))+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((px)*(px))))+(((0.0280000000000200)*(px)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=py;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=((-0.500000000000000)*(3.14159265358979));
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
                continue;
            }
            }
            }
            }
            }
            }
            if( numsolutionsj4 == 0 ) { continue; }
            if( j4valid[0] && j4valid[1] && IKabs(cj4array[0]-cj4array[1]) < 0.0001 && IKabs(sj4array[0]-sj4array[1]) < 0.0001 )
            j4valid[1]=false;
        for(int ij4 = 0; ij4 < numsolutionsj4; ++ij4)
        {
        if( !j4valid[ij4] ) { continue; }
        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
        
        {
        IKReal evalcond[12]; int numsolutionsj5 = 0;
        IKReal j5array[1], cj5array[1], sj5array[1];
        bool j5valid[1]={false};
            {
            evalcond[0]=(((((0.0140000000000100)+(((-1.00000000000000)*(px)))))*(((0.0140000000000100)+(((-1.00000000000000)*(px))))))+((((0.0329999999999018)+(pz)))*(((0.0329999999999018)+(pz)))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x163=((0.0329999999999018)+(pz));
                IKReal x164=((0.0140000000000100)+(((-1.00000000000000)*(px))));
                IKReal x165=(x163)*(x163);
                IKReal x166=(x164)*(x164);
                IKReal x167=((x166)+(x165));
                IKReal x168=((IKabs(x167) != 0)?((IKReal)1/(x167)):(IKReal)1.0e30);
                IKReal x169=((0.149999999999636)*(cj3));
                IKReal x170=((0.155000000000655)+(x169));
                j5array[0]=IKatan2(((x168)*(((((0.149999999999636)*(sj3)*(sj4)*(x163)))+(((x164)*(x170)))))), ((x168)*(((((0.149999999999636)*(sj3)*(sj4)*(x164)))+(((-1.00000000000000)*(x163)*(x170)))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            evalcond[0]=((((((0.155000000000655)+(((0.149999999999636)*(cj3)))))*(((0.0140000000000100)+(((-1.00000000000000)*(px)))))))+(((-0.149999999999636)*(sj3)*(sj4)*(((0.0329999999999018)+(pz))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x171=((0.149999999999636)*(cj3));
                IKReal x172=((0.155000000000655)+(x171));
                IKReal x173=((0.0329999999999018)+(pz));
                IKReal x174=((0.0140000000000100)+(((-1.00000000000000)*(px))));
                IKReal x175=((x172)*(x174));
                IKReal x176=((0.149999999999636)*(sj3)*(sj4)*(x173));
                IKReal x177=((x175)+(((-1.00000000000000)*(x176))));
                IKReal x178=((IKabs(x177) != 0)?((IKReal)1/(x177)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x178)*(((((-1.00000000000000)*((x173)*(x173))))+((x172)*(x172))))), ((x178)*(((((-1.00000000000000)*(x174)*(((0.0329999999999018)+(pz)))))+(((0.149999999999636)*(sj3)*(sj4)*(x172)))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            evalcond[0]=((((((0.0329999999999018)+(pz)))*(((0.155000000000655)+(((0.149999999999636)*(cj3)))))))+(((0.149999999999636)*(sj3)*(sj4)*(((0.0140000000000100)+(((-1.00000000000000)*(px))))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x179=((0.0329999999999018)+(pz));
                IKReal x180=((0.0140000000000100)+(((-1.00000000000000)*(px))));
                IKReal x181=((0.149999999999636)*(cj3));
                IKReal x182=((0.155000000000655)+(x181));
                IKReal x183=((0.149999999999636)*(sj3)*(sj4)*(x180));
                IKReal x184=((x179)*(x182));
                IKReal x185=((x184)+(x183));
                IKReal x186=((IKabs(x185) != 0)?((IKReal)1/(x185)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x186)*(((((0.149999999999636)*(sj3)*(sj4)*(x182)))+(((x179)*(x180)))))), ((x186)*(((((-1.00000000000000)*((x182)*(x182))))+((x180)*(x180))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            evalcond[0]=((((((0.155000000000655)+(((0.149999999999636)*(cj3)))))*(((0.0140000000000100)+(((-1.00000000000000)*(px)))))))+(((-0.149999999999636)*(sj3)*(sj4)*(((0.0329999999999018)+(pz))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x187=((0.0140000000000100)+(((-1.00000000000000)*(px))));
                IKReal x188=((0.149999999999636)*(cj3));
                IKReal x189=((0.155000000000655)+(x188));
                IKReal x190=((x187)*(x189));
                IKReal x191=((0.0329999999999018)+(pz));
                IKReal x192=((0.149999999999636)*(sj3)*(sj4)*(x191));
                IKReal x193=((((-1.00000000000000)*(x192)))+(x190));
                IKReal x194=((IKabs(x193) != 0)?((IKReal)1/(x193)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x194)*(((((-0.0224999999998090)*((sj3)*(sj3))*((sj4)*(sj4))))+((x187)*(x187))))), ((x194)*(((((0.149999999999636)*(sj3)*(sj4)*(x189)))+(((-1.00000000000000)*(x187)*(x191)))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x195=((0.149999999999636)*(cj3));
            IKReal x196=((0.149999999999636)*(sj3)*(sj4));
            IKReal x197=((0.0280000000000200)*(px));
            IKReal x198=(px)*(px);
            IKReal x199=(pz)*(pz);
            IKReal x200=(py)*(py);
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.188000000000102)+(x195)+(pz));
            evalcond[2]=((0.0452399999999216)+(((-0.0659999999998035)*(pz)))+(((-1.00000000000000)*(x200)))+(x197)+(((0.310000000001310)*(x195)))+(((-1.00000000000000)*(x198)))+(((-1.00000000000000)*(x199))));
            evalcond[3]=((0.0140000000000100)+(((-1.00000000000000)*(px)))+(((-1.00000000000000)*(x196))));
            evalcond[4]=((-0.188000000000102)+(((-1.00000000000000)*(pz)))+(((-1.00000000000000)*(x195))));
            evalcond[5]=((-0.0140000000000100)+(x196)+(px));
            evalcond[6]=((((-1.00000000000000)*(py)*(sj4)))+(((0.0140000000000100)*(cj4)))+(((-35.7142857140861)*(cj4)*(x197))));
            evalcond[7]=((((-1.00000000000000)*(cj4)*(py)))+(((35.7142857140861)*(sj4)*(x197)))+(((-0.0140000000000100)*(sj4)))+(((0.149999999999636)*(sj3))));
            evalcond[8]=((-0.0130400000003874)+(((-1.00000000000000)*(x200)))+(((-0.376000000000204)*(pz)))+(x197)+(((-1.00000000000000)*(x198)))+(((-1.00000000000000)*(x199))));
            evalcond[9]=((0.0580399999998917)+(((-1.00000000000000)*(x200)))+(((-0.0279999999997926)*(x196)))+(((0.376000000000204)*(x195)))+(((-1.00000000000000)*(x198)))+(((-1.00000000000000)*(x199))));
            evalcond[10]=((0.149999999999636)+(((1.25333333334129)*(x195)))+(((238.095238095149)*(x196)*(x197)))+(((-0.0933333333332484)*(x196)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((6.66666666668607)*(pz)*(x195))));
            evalcond[11]=((((-1.00000000000000)*(pz)*(sj3)))+(((-6.66666666668607)*(cj4)*(py)*(x195)))+(((238.095238095149)*(sj4)*(x195)*(x197)))+(((-0.188000000000102)*(sj3)))+(((-0.0933333333332484)*(sj4)*(x195))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=0;
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x201=((0.149999999999636)*(cj3));
            IKReal x202=((0.149999999999636)*(sj3)*(sj4));
            IKReal x203=((-0.0329999999999018)+(x202));
            IKReal x204=((((-1.00000000000000)*(pz)))+(x203));
            IKReal x205=(px)*(px);
            IKReal x206=(pz)*(pz);
            IKReal x207=((0.0659999999998035)*(pz));
            IKReal x208=(py)*(py);
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.141000000001441)+(x201)+(px));
            evalcond[2]=((0.0452399999999216)+(((-1.00000000000000)*(x205)))+(((0.0280000000000200)*(px)))+(((0.310000000001310)*(x201)))+(((-1.00000000000000)*(x207)))+(((-1.00000000000000)*(x208)))+(((-1.00000000000000)*(x206))));
            evalcond[3]=((-0.141000000001441)+(((-1.00000000000000)*(px)))+(((-1.00000000000000)*(x201))));
            evalcond[4]=x204;
            evalcond[5]=x204;
            evalcond[6]=((((-1.00000000000000)*(py)*(sj4)))+(((0.0329999999999018)*(cj4)))+(((15.1515151516069)*(cj4)*(x207))));
            evalcond[7]=((((-15.1515151516069)*(sj4)*(x207)))+(((-1.00000000000000)*(cj4)*(py)))+(((0.149999999999636)*(sj3)))+(((-0.0329999999999018)*(sj4))));
            evalcond[8]=((0.00152999999960457)+(((-1.00000000000000)*(x205)))+(((-0.282000000002881)*(px)))+(((-1.00000000000000)*(x207)))+(((-1.00000000000000)*(x208)))+(((-1.00000000000000)*(x206))));
            evalcond[9]=((0.0434700000000703)+(((0.282000000002881)*(x201)))+(((-0.0659999999998035)*(x202)))+(((-1.00000000000000)*(x205)))+(((-1.00000000000000)*(x208)))+(((-1.00000000000000)*(x206))));
            evalcond[10]=((0.149999999999636)+(((6.66666666668607)*(px)*(x201)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((-0.219999999999345)*(x202)))+(((0.940000000009604)*(x201)))+(((-101.010101011023)*(x202)*(x207))));
            evalcond[11]=((((-1.00000000000000)*(px)*(sj3)))+(((-0.141000000001441)*(sj3)))+(((-0.219999999999345)*(sj4)*(x201)))+(((-6.66666666668607)*(cj4)*(py)*(x201)))+(((-101.010101011023)*(sj4)*(x201)*(x207))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=((0.500000000000000)*(3.14159265358979));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x209=((0.149999999999636)*(cj3));
            IKReal x210=((0.122000000001208)+(x209));
            IKReal x211=((((-1.00000000000000)*(pz)))+(x210));
            IKReal x212=((0.149999999999636)*(sj3)*(sj4));
            IKReal x213=((0.0140000000000100)+(x212));
            IKReal x214=((((-1.00000000000000)*(px)))+(x213));
            IKReal x215=((0.0280000000000200)*(px));
            IKReal x216=(px)*(px);
            IKReal x217=(pz)*(pz);
            IKReal x218=(py)*(py);
            IKReal x219=((x216)+(x217)+(x218));
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=x211;
            evalcond[2]=((0.0452399999999216)+(((-0.0659999999998035)*(pz)))+(x215)+(((-1.00000000000000)*(x219)))+(((0.310000000001310)*(x209))));
            evalcond[3]=x214;
            evalcond[4]=x211;
            evalcond[5]=x214;
            evalcond[6]=((((-1.00000000000000)*(py)*(sj4)))+(((-0.0140000000000100)*(cj4)))+(((35.7142857140861)*(cj4)*(x215))));
            evalcond[7]=((((0.0140000000000100)*(sj4)))+(((-1.00000000000000)*(cj4)*(py)))+(((-35.7142857140861)*(sj4)*(x215)))+(((0.149999999999636)*(sj3))));
            evalcond[8]=((0.00741999999956988)+(((0.244000000002416)*(pz)))+(x215)+(((-1.00000000000000)*(x219))));
            evalcond[9]=((0.0375800000001618)+(((0.244000000000597)*(x209)))+(((0.0279999999997926)*(x212)))+(((-1.00000000000000)*(x219))));
            evalcond[10]=((0.149999999999636)+(((0.0933333333332484)*(x212)))+(((-6.66666666668607)*(pz)*(x209)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((0.813333333346236)*(x209)))+(((-238.095238095149)*(x212)*(x215))));
            evalcond[11]=((((-0.122000000001208)*(sj3)))+(((pz)*(sj3)))+(((-6.66666666668607)*(cj4)*(py)*(x209)))+(((0.0933333333332484)*(sj4)*(x209)))+(((-238.095238095149)*(sj4)*(x209)*(x215))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=3.14159265358979;
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x220=((0.149999999999636)*(cj3));
            IKReal x221=((0.168999999999869)+(x220));
            IKReal x222=((((-1.00000000000000)*(px)))+(x221));
            IKReal x223=((0.149999999999636)*(sj3)*(sj4));
            IKReal x224=(px)*(px);
            IKReal x225=(pz)*(pz);
            IKReal x226=((0.0659999999998035)*(pz));
            IKReal x227=(py)*(py);
            IKReal x228=((x225)+(x224)+(x227)+(x226));
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=x222;
            evalcond[2]=((0.0452399999999216)+(((0.310000000001310)*(x220)))+(((0.0280000000000200)*(px)))+(((-1.00000000000000)*(x228))));
            evalcond[3]=x222;
            evalcond[4]=((-0.0329999999999018)+(((-1.00000000000000)*(pz)))+(((-1.00000000000000)*(x223))));
            evalcond[5]=((0.0329999999999018)+(x223)+(pz));
            evalcond[6]=((((-1.00000000000000)*(py)*(sj4)))+(((-15.1515151516069)*(cj4)*(x226)))+(((-0.0329999999999018)*(cj4))));
            evalcond[7]=((((0.0329999999999018)*(sj4)))+(((15.1515151516069)*(sj4)*(x226)))+(((-1.00000000000000)*(cj4)*(py)))+(((0.149999999999636)*(sj3))));
            evalcond[8]=((-0.00715000000042210)+(((0.337999999999738)*(px)))+(((-1.00000000000000)*(x228))));
            evalcond[9]=((0.0521499999999833)+(((-1.00000000000000)*(x224)))+(((0.337999999999738)*(x220)))+(((0.0659999999998035)*(x223)))+(((-1.00000000000000)*(x225)))+(((-1.00000000000000)*(x227))));
            evalcond[10]=((0.149999999999636)+(((-6.66666666668607)*(px)*(x220)))+(((1.12666666666337)*(x220)))+(((0.219999999999345)*(x223)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((101.010101011023)*(x223)*(x226))));
            evalcond[11]=((((-6.66666666668607)*(cj4)*(py)*(x220)))+(((-0.168999999999869)*(sj3)))+(((0.219999999999345)*(sj4)*(x220)))+(((101.010101011023)*(sj4)*(x220)*(x226)))+(((px)*(sj3))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=((-0.500000000000000)*(3.14159265358979));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
                continue;
            }
            }
            }
            }
            }
            }
            }
            }
            }
            if( numsolutionsj5 == 0 ) { continue; }
            for(int ij5 = 0; ij5 < numsolutionsj5; ++ij5)
        {
        if( !j5valid[ij5] ) { continue; }
        j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
        
        new_r00=((((cj3)*(cj4)*(r10)))+(((r00)*(sj3)*(sj5)))+(((-1.00000000000000)*(cj3)*(cj5)*(r00)*(sj4)))+(((cj5)*(r20)*(sj3)))+(((cj3)*(r20)*(sj4)*(sj5))));
        new_r01=((((cj5)*(r21)*(sj3)))+(((cj3)*(r21)*(sj4)*(sj5)))+(((-1.00000000000000)*(cj3)*(cj5)*(r01)*(sj4)))+(((cj3)*(cj4)*(r11)))+(((r01)*(sj3)*(sj5))));
        new_r02=((((r02)*(sj3)*(sj5)))+(((cj5)*(r22)*(sj3)))+(((-1.00000000000000)*(cj3)*(cj5)*(r02)*(sj4)))+(((cj3)*(cj4)*(r12)))+(((cj3)*(r22)*(sj4)*(sj5))));
        new_r10=((((cj4)*(cj5)*(r00)))+(((r10)*(sj4)))+(((-1.00000000000000)*(cj4)*(r20)*(sj5))));
        new_r11=((((cj4)*(cj5)*(r01)))+(((-1.00000000000000)*(cj4)*(r21)*(sj5)))+(((r11)*(sj4))));
        new_r12=((((r12)*(sj4)))+(((-1.00000000000000)*(cj4)*(r22)*(sj5)))+(((cj4)*(cj5)*(r02))));
        new_r20=((((r20)*(sj3)*(sj4)*(sj5)))+(((-1.00000000000000)*(cj3)*(cj5)*(r20)))+(((-1.00000000000000)*(cj3)*(r00)*(sj5)))+(((cj4)*(r10)*(sj3)))+(((-1.00000000000000)*(cj5)*(r00)*(sj3)*(sj4))));
        new_r21=((((cj4)*(r11)*(sj3)))+(((-1.00000000000000)*(cj3)*(r01)*(sj5)))+(((-1.00000000000000)*(cj5)*(r01)*(sj3)*(sj4)))+(((r21)*(sj3)*(sj4)*(sj5)))+(((-1.00000000000000)*(cj3)*(cj5)*(r21))));
        new_r22=((((cj4)*(r12)*(sj3)))+(((-1.00000000000000)*(cj3)*(cj5)*(r22)))+(((-1.00000000000000)*(cj5)*(r02)*(sj3)*(sj4)))+(((-1.00000000000000)*(cj3)*(r02)*(sj5)))+(((r22)*(sj3)*(sj4)*(sj5))));
        {
        IKReal j1eval;
        j1eval=new_r21;
        if( j1eval >= 0.999990 && j1eval <= 1.000010 )
        {
            {
            j1 = -1.570796; sj1 = -1.000000; cj1 = 0.000000;
            IKReal j0mul = 1;
            j0=0;
            IKReal j2mul=1.00000000000000;
            j2=((-1.00000000000000)*(IKatan2(new_r10, new_r00)));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[0].fmul = j0mul;
            solution.basesol[0].freeind = 0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[2].fmul = j2mul;
            solution.basesol[2].freeind = 0;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.vfree.resize(1);
            solution.vfree[0] = 0;
            }
            
        } else
        if( j1eval >= -1.000010 && j1eval <= -0.999990 )
        {
            {
            j1 = 1.570796; sj1 = 1.000000; cj1 = 0.000000;
            IKReal j0mul = 1;
            j0=0;
            IKReal j2mul=-1.00000000000000;
            j2=((((-1.00000000000000)*(IKatan2(new_r02, ((-1.00000000000000)*(new_r12))))))+(((-1.00000000000000)*(3.14159265358979))));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[0].fmul = j0mul;
            solution.basesol[0].freeind = 0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[2].fmul = j2mul;
            solution.basesol[2].freeind = 0;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.vfree.resize(1);
            solution.vfree[0] = 0;
            }
            
        } else
        {
            {
            IKReal j0array[2], cj0array[2], sj0array[2];
            bool j0valid[2]={false};
            j0array[0]=IKatan2(((-1.00000000000000)*(new_r20)), new_r22);
            sj0array[0]=IKsin(j0array[0]);
            cj0array[0]=IKcos(j0array[0]);
            j0array[1] = j0array[0] > 0 ? j0array[0]-IKPI : j0array[0]+IKPI;
            sj0array[1] = -sj0array[0];
            cj0array[1] = -cj0array[0];
            if( j0array[0] > IKPI )
                j0array[0]-=IK2PI;
            else if( j0array[0] < -IKPI )
                j0array[0]+=IK2PI;
            j0valid[0] = true;
            if( j0array[1] > IKPI )
                j0array[1]-=IK2PI;
            else if( j0array[1] < -IKPI )
                j0array[1]+=IK2PI;
            j0valid[1] = true;
            if( j0valid[0] && j0valid[1] && IKabs(cj0array[0]-cj0array[1]) < 0.0001 && IKabs(sj0array[0]-sj0array[1]) < 0.0001 )
                j0valid[1]=false;
            for(int ij0 = 0; ij0 < 2; ++ij0)
            {
            if( !j0valid[ij0] ) { continue; }
            j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
            
            {
            IKReal j1array[1], cj1array[1], sj1array[1];
            bool j1valid[1]={false};
            j1array[0]=IKatan2(((-1.00000000000000)*(new_r21)), ((new_r22)*(((IKabs(cj0) != 0)?((IKReal)1/(cj0)):(IKReal)1.0e30))));
            sj1array[0]=IKsin(j1array[0]);
            cj1array[0]=IKcos(j1array[0]);
            if( j1array[0] > IKPI )
                j1array[0]-=IK2PI;
            else if( j1array[0] < -IKPI )
                j1array[0]+=IK2PI;
            j1valid[0] = true;
            for(int ij1 = 0; ij1 < 1; ++ij1)
            {
            if( !j1valid[ij1] ) { continue; }
            j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
            
            {
            IKReal j2array[1], cj2array[1], sj2array[1];
            bool j2valid[1]={false};
            IKReal x229=((IKabs(cj1) != 0)?((IKReal)1/(cj1)):(IKReal)1.0e30);
            j2array[0]=IKatan2(((new_r01)*(x229)), ((new_r11)*(x229)));
            sj2array[0]=IKsin(j2array[0]);
            cj2array[0]=IKcos(j2array[0]);
            if( j2array[0] > IKPI )
                j2array[0]-=IK2PI;
            else if( j2array[0] < -IKPI )
                j2array[0]+=IK2PI;
            j2valid[0] = true;
            for(int ij2 = 0; ij2 < 1; ++ij2)
            {
            if( !j2valid[ij2] ) { continue; }
            j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
            
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.vfree.resize(0);
            }
            }
            }
            }
            }
            }
            
        }
        }
        }
        }
        }
        }
        }
        }
        
    } else
    {
        evalcond=((j6)+(((0.500000000000000)*(3.14159265358979))));
        if( IKabs(evalcond) < 0.00001 )
        {
        {
        IKReal j3array[2], cj3array[2], sj3array[2];
        bool j3valid[2]={false};
        cj3array[0]=((-0.972903225803748)+(((-0.602150537633861)*(pz)))+(((-1.41935483871202)*(px)))+(((21.5053763440810)*((py)*(py))))+(((21.5053763440810)*((pz)*(pz))))+(((21.5053763440810)*((px)*(px)))));
        if( cj3array[0] >= -1.0001 && cj3array[0] <= 1.0001 )
        {
            j3valid[0] = j3valid[1] = true;
            j3array[0] = IKacos(cj3array[0]);
            sj3array[0] = IKsin(j3array[0]);
            cj3array[1] = cj3array[0];
            j3array[1] = -j3array[0];
            sj3array[1] = -sj3array[0];
        }
        else if( isnan(cj3array[0]) )
        {
            // probably any value will work
            j3valid[0] = true;
            cj3array[0] = 1; sj3array[0] = 0; j3array[0] = 0;
        }
        if( j3valid[0] && j3valid[1] && IKabs(cj3array[0]-cj3array[1]) < 0.0001 && IKabs(sj3array[0]-sj3array[1]) < 0.0001 )
            j3valid[1]=false;
        for(int ij3 = 0; ij3 < 2; ++ij3)
        {
        if( !j3valid[ij3] ) { continue; }
        j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
        
        {
        IKReal evalcond[3]; int numsolutionsj4 = 0;
        IKReal j4array[2], cj4array[2], sj4array[2];
        bool j4valid[2]={false};
            {
            evalcond[0]=sj3;
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                cj4array[0]=((6.66666666668607)*(py)*(((IKabs(sj3) != 0)?((IKReal)1/(sj3)):(IKReal)1.0e30)));
                if( cj4array[0] >= -1.0001 && cj4array[0] <= 1.0001 )
                {
                    j4valid[0] = j4valid[1] = true;
                    j4array[0] = IKacos(cj4array[0]);
                    sj4array[0] = IKsin(j4array[0]);
                    cj4array[1] = cj4array[0];
                    j4array[1] = -j4array[0];
                    sj4array[1] = -sj4array[0];
                }
                else if( isnan(cj4array[0]) )
                {
                    // probably any value will work
                    j4valid[0] = true;
                    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
                }
                numsolutionsj4 = 2;
                
            } else
            {
            IKReal x230=((0.149999999999636)*(sj3));
            IKReal x231=((((-1.00000000000000)*(py)))+(x230));
            evalcond[0]=x231;
            evalcond[1]=((0.0452399999999216)+(((0.0659999999998035)*(px)))+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((0.0280000000000200)*(pz)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x231;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=0;
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            IKReal x232=((-1.00000000000000)*(py));
            evalcond[0]=x232;
            evalcond[1]=((0.0452399999999216)+(((0.0659999999998035)*(px)))+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((0.0280000000000200)*(pz)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x232;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=((0.500000000000000)*(3.14159265358979));
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            IKReal x233=((0.149999999999636)*(sj3));
            IKReal x234=((x233)+(py));
            evalcond[0]=((-1.00000000000000)*(x234));
            evalcond[1]=((0.0452399999999216)+(((0.0659999999998035)*(px)))+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((0.0280000000000200)*(pz)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x234;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=3.14159265358979;
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            evalcond[0]=((-1.00000000000000)*(py));
            evalcond[1]=((0.0452399999999216)+(((0.0659999999998035)*(px)))+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((0.0280000000000200)*(pz)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=py;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=((-0.500000000000000)*(3.14159265358979));
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
                continue;
            }
            }
            }
            }
            }
            }
            if( numsolutionsj4 == 0 ) { continue; }
            if( j4valid[0] && j4valid[1] && IKabs(cj4array[0]-cj4array[1]) < 0.0001 && IKabs(sj4array[0]-sj4array[1]) < 0.0001 )
            j4valid[1]=false;
        for(int ij4 = 0; ij4 < numsolutionsj4; ++ij4)
        {
        if( !j4valid[ij4] ) { continue; }
        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
        
        {
        IKReal evalcond[12]; int numsolutionsj5 = 0;
        IKReal j5array[1], cj5array[1], sj5array[1];
        bool j5valid[1]={false};
            {
            evalcond[0]=(((((0.0329999999999018)+(((-1.00000000000000)*(px)))))*(((0.0329999999999018)+(((-1.00000000000000)*(px))))))+((((0.0140000000000100)+(((-1.00000000000000)*(pz)))))*(((0.0140000000000100)+(((-1.00000000000000)*(pz)))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x235=((0.0329999999999018)+(((-1.00000000000000)*(px))));
                IKReal x236=((0.0140000000000100)+(((-1.00000000000000)*(pz))));
                IKReal x237=(x236)*(x236);
                IKReal x238=(x235)*(x235);
                IKReal x239=((x238)+(x237));
                IKReal x240=((IKabs(x239) != 0)?((IKReal)1/(x239)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x240)*(((((x236)*(((0.155000000000655)+(((0.149999999999636)*(cj3)))))))+(((0.149999999999636)*(sj3)*(sj4)*(x235)))))), ((x240)*(((((-1.00000000000000)*(x235)*(((0.155000000000655)+(((0.149999999999636)*(cj3)))))))+(((0.149999999999636)*(sj3)*(sj4)*(x236)))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            evalcond[0]=((((((0.155000000000655)+(((0.149999999999636)*(cj3)))))*(((0.0140000000000100)+(((-1.00000000000000)*(pz)))))))+(((-0.149999999999636)*(sj3)*(sj4)*(((0.0329999999999018)+(((-1.00000000000000)*(px))))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x241=((0.0140000000000100)+(((-1.00000000000000)*(pz))));
                IKReal x242=((0.149999999999636)*(cj3));
                IKReal x243=((0.155000000000655)+(x242));
                IKReal x244=((x241)*(x243));
                IKReal x245=((0.0329999999999018)+(((-1.00000000000000)*(px))));
                IKReal x246=((0.149999999999636)*(sj3)*(sj4)*(x245));
                IKReal x247=((x244)+(((-1.00000000000000)*(x246))));
                IKReal x248=((IKabs(x247) != 0)?((IKReal)1/(x247)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x248)*(((((-0.0224999999998090)*((sj3)*(sj3))*((sj4)*(sj4))))+((x241)*(x241))))), ((x248)*(((((0.149999999999636)*(sj3)*(sj4)*(x243)))+(((-1.00000000000000)*(x241)*(((0.0329999999999018)+(((-1.00000000000000)*(px)))))))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            evalcond[0]=((((0.149999999999636)*(sj3)*(sj4)*(((0.0140000000000100)+(((-1.00000000000000)*(pz)))))))+(((((0.0329999999999018)+(((-1.00000000000000)*(px)))))*(((0.155000000000655)+(((0.149999999999636)*(cj3))))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x249=((0.149999999999636)*(cj3));
                IKReal x250=((0.155000000000655)+(x249));
                IKReal x251=((0.0329999999999018)+(((-1.00000000000000)*(px))));
                IKReal x252=((0.0140000000000100)+(((-1.00000000000000)*(pz))));
                IKReal x253=((0.149999999999636)*(sj3)*(sj4)*(x252));
                IKReal x254=((x250)*(x251));
                IKReal x255=((x254)+(x253));
                IKReal x256=((IKabs(x255) != 0)?((IKReal)1/(x255)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x256)*(((((0.149999999999636)*(sj3)*(sj4)*(x250)))+(((x251)*(x252)))))), ((x256)*(((((-1.00000000000000)*((x251)*(x251))))+(((0.0224999999998090)*((sj3)*(sj3))*((sj4)*(sj4))))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            evalcond[0]=((((0.149999999999636)*(sj3)*(sj4)*(((0.0329999999999018)+(((-1.00000000000000)*(px)))))))+(((-1.00000000000000)*(((0.155000000000655)+(((0.149999999999636)*(cj3)))))*(((0.0140000000000100)+(((-1.00000000000000)*(pz))))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x257=((0.0329999999999018)+(((-1.00000000000000)*(px))));
                IKReal x258=((0.149999999999636)*(cj3));
                IKReal x259=((0.155000000000655)+(x258));
                IKReal x260=((0.0140000000000100)+(((-1.00000000000000)*(pz))));
                IKReal x261=((0.149999999999636)*(sj3)*(sj4)*(x257));
                IKReal x262=((x259)*(x260));
                IKReal x263=((((-1.00000000000000)*(x262)))+(x261));
                IKReal x264=((IKabs(x263) != 0)?((IKReal)1/(x263)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x264)*((((x257)*(x257))+(((-1.00000000000000)*((x259)*(x259))))))), ((x264)*(((((x257)*(x260)))+(((-0.149999999999636)*(sj3)*(sj4)*(x259)))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x265=((0.149999999999636)*(cj3));
            IKReal x266=((0.188000000000102)+(x265));
            IKReal x267=((((-1.00000000000000)*(px)))+(x266));
            IKReal x268=((0.149999999999636)*(sj3)*(sj4));
            IKReal x269=((0.0280000000000200)*(pz));
            IKReal x270=(px)*(px);
            IKReal x271=(pz)*(pz);
            IKReal x272=(py)*(py);
            IKReal x273=((x272)+(x270)+(x271));
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.0452399999999216)+(((0.310000000001310)*(x265)))+(((0.0659999999998035)*(px)))+(x269)+(((-1.00000000000000)*(x273))));
            evalcond[2]=x267;
            evalcond[3]=x267;
            evalcond[4]=((0.0140000000000100)+(((-1.00000000000000)*(pz)))+(((-1.00000000000000)*(x268))));
            evalcond[5]=((-0.0140000000000100)+(pz)+(x268));
            evalcond[6]=((((-35.7142857140861)*(cj4)*(x269)))+(((-1.00000000000000)*(py)*(sj4)))+(((0.0140000000000100)*(cj4))));
            evalcond[7]=((((35.7142857140861)*(sj4)*(x269)))+(((-1.00000000000000)*(cj4)*(py)))+(((-0.0140000000000100)*(sj4)))+(((0.149999999999636)*(sj3))));
            evalcond[8]=((-0.0130400000003874)+(((0.376000000000204)*(px)))+(x269)+(((-1.00000000000000)*(x273))));
            evalcond[9]=((0.0580399999998917)+(((0.376000000000204)*(x265)))+(((-0.0279999999997926)*(x268)))+(((-1.00000000000000)*(x273))));
            evalcond[10]=((((-0.0933333333332484)*(sj4)*(x265)))+(((238.095238095149)*(sj4)*(x265)*(x269)))+(((-0.188000000000102)*(sj3)))+(((-6.66666666668607)*(cj4)*(py)*(x265)))+(((px)*(sj3))));
            evalcond[11]=((0.149999999999636)+(((-0.0933333333332484)*(x268)))+(((1.25333333334129)*(x265)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((238.095238095149)*(x268)*(x269)))+(((-6.66666666668607)*(px)*(x265))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=0;
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x274=((0.149999999999636)*(cj3));
            IKReal x275=((0.149999999999636)*(sj3)*(sj4));
            IKReal x276=((0.0659999999998035)*(px));
            IKReal x277=(px)*(px);
            IKReal x278=(pz)*(pz);
            IKReal x279=(py)*(py);
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.0452399999999216)+(((-1.00000000000000)*(x278)))+(((0.0280000000000200)*(pz)))+(((0.310000000001310)*(x274)))+(((-1.00000000000000)*(x277)))+(x276)+(((-1.00000000000000)*(x279))));
            evalcond[2]=((0.141000000001441)+(x274)+(pz));
            evalcond[3]=((0.0329999999999018)+(((-1.00000000000000)*(px)))+(((-1.00000000000000)*(x275))));
            evalcond[4]=((-0.141000000001441)+(((-1.00000000000000)*(pz)))+(((-1.00000000000000)*(x274))));
            evalcond[5]=((-0.0329999999999018)+(x275)+(px));
            evalcond[6]=((((-15.1515151516069)*(cj4)*(x276)))+(((-1.00000000000000)*(py)*(sj4)))+(((0.0329999999999018)*(cj4))));
            evalcond[7]=((((-1.00000000000000)*(cj4)*(py)))+(((15.1515151516069)*(sj4)*(x276)))+(((0.149999999999636)*(sj3)))+(((-0.0329999999999018)*(sj4))));
            evalcond[8]=((0.00152999999960457)+(((-1.00000000000000)*(x278)))+(((-0.282000000002881)*(pz)))+(((-1.00000000000000)*(x277)))+(x276)+(((-1.00000000000000)*(x279))));
            evalcond[9]=((0.0434700000000703)+(((-0.0659999999998035)*(x275)))+(((-1.00000000000000)*(x278)))+(((0.282000000002881)*(x274)))+(((-1.00000000000000)*(x277)))+(((-1.00000000000000)*(x279))));
            evalcond[10]=((((-1.00000000000000)*(pz)*(sj3)))+(((-6.66666666668607)*(cj4)*(py)*(x274)))+(((-0.219999999999345)*(sj4)*(x274)))+(((-0.141000000001441)*(sj3)))+(((101.010101011023)*(sj4)*(x274)*(x276))));
            evalcond[11]=((0.149999999999636)+(((6.66666666668607)*(pz)*(x274)))+(((0.940000000009604)*(x274)))+(((101.010101011023)*(x275)*(x276)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((-0.219999999999345)*(x275))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=((0.500000000000000)*(3.14159265358979));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x280=((0.149999999999636)*(cj3));
            IKReal x281=((0.149999999999636)*(sj3)*(sj4));
            IKReal x282=((0.0140000000000100)+(x281));
            IKReal x283=((((-1.00000000000000)*(pz)))+(x282));
            IKReal x284=((0.0280000000000200)*(pz));
            IKReal x285=(px)*(px);
            IKReal x286=(pz)*(pz);
            IKReal x287=(py)*(py);
            IKReal x288=((x287)+(x286)+(x285));
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.0452399999999216)+(((-1.00000000000000)*(x288)))+(x284)+(((0.0659999999998035)*(px)))+(((0.310000000001310)*(x280))));
            evalcond[2]=((0.122000000001208)+(x280)+(px));
            evalcond[3]=((-0.122000000001208)+(((-1.00000000000000)*(px)))+(((-1.00000000000000)*(x280))));
            evalcond[4]=x283;
            evalcond[5]=x283;
            evalcond[6]=((((35.7142857140861)*(cj4)*(x284)))+(((-1.00000000000000)*(py)*(sj4)))+(((-0.0140000000000100)*(cj4))));
            evalcond[7]=((((0.0140000000000100)*(sj4)))+(((-1.00000000000000)*(cj4)*(py)))+(((0.149999999999636)*(sj3)))+(((-35.7142857140861)*(sj4)*(x284))));
            evalcond[8]=((0.00741999999956988)+(((-1.00000000000000)*(x288)))+(x284)+(((-0.244000000002416)*(px))));
            evalcond[9]=((0.0375800000001618)+(((-1.00000000000000)*(x288)))+(((0.244000000000597)*(x280)))+(((0.0279999999997926)*(x281))));
            evalcond[10]=((((-0.122000000001208)*(sj3)))+(((-1.00000000000000)*(px)*(sj3)))+(((-238.095238095149)*(sj4)*(x280)*(x284)))+(((-6.66666666668607)*(cj4)*(py)*(x280)))+(((0.0933333333332484)*(sj4)*(x280))));
            evalcond[11]=((0.149999999999636)+(((0.0933333333332484)*(x281)))+(((6.66666666668607)*(px)*(x280)))+(((0.813333333346236)*(x280)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((-238.095238095149)*(x281)*(x284))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=3.14159265358979;
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x289=((0.149999999999636)*(cj3));
            IKReal x290=((0.168999999999869)+(x289));
            IKReal x291=((((-1.00000000000000)*(pz)))+(x290));
            IKReal x292=((0.149999999999636)*(sj3)*(sj4));
            IKReal x293=((0.0329999999999018)+(x292));
            IKReal x294=((((-1.00000000000000)*(px)))+(x293));
            IKReal x295=((0.0659999999998035)*(px));
            IKReal x296=(px)*(px);
            IKReal x297=(pz)*(pz);
            IKReal x298=(py)*(py);
            IKReal x299=((x298)+(x296)+(x297));
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.0452399999999216)+(((0.310000000001310)*(x289)))+(x295)+(((0.0280000000000200)*(pz)))+(((-1.00000000000000)*(x299))));
            evalcond[2]=x291;
            evalcond[3]=x294;
            evalcond[4]=x291;
            evalcond[5]=x294;
            evalcond[6]=((((-1.00000000000000)*(py)*(sj4)))+(((15.1515151516069)*(cj4)*(x295)))+(((-0.0329999999999018)*(cj4))));
            evalcond[7]=((((0.0329999999999018)*(sj4)))+(((-15.1515151516069)*(sj4)*(x295)))+(((-1.00000000000000)*(cj4)*(py)))+(((0.149999999999636)*(sj3))));
            evalcond[8]=((-0.00715000000042210)+(x295)+(((-1.00000000000000)*(x299)))+(((0.337999999999738)*(pz))));
            evalcond[9]=((0.0521499999999833)+(((0.0659999999998035)*(x292)))+(((0.337999999999738)*(x289)))+(((-1.00000000000000)*(x299))));
            evalcond[10]=((((-0.168999999999869)*(sj3)))+(((pz)*(sj3)))+(((-6.66666666668607)*(cj4)*(py)*(x289)))+(((-101.010101011023)*(sj4)*(x289)*(x295)))+(((0.219999999999345)*(sj4)*(x289))));
            evalcond[11]=((0.149999999999636)+(((-101.010101011023)*(x292)*(x295)))+(((-6.66666666668607)*(pz)*(x289)))+(((1.12666666666337)*(x289)))+(((0.219999999999345)*(x292)))+(((-1.00000000000000)*(cj4)*(py)*(sj3))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  )
            {
                j5array[0]=((-0.500000000000000)*(3.14159265358979));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
                continue;
            }
            }
            }
            }
            }
            }
            }
            }
            }
            if( numsolutionsj5 == 0 ) { continue; }
            for(int ij5 = 0; ij5 < numsolutionsj5; ++ij5)
        {
        if( !j5valid[ij5] ) { continue; }
        j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
        
        new_r00=((((r20)*(sj3)*(sj5)))+(((-1.00000000000000)*(cj3)*(r00)*(sj4)*(sj5)))+(((cj3)*(cj4)*(r10)))+(((-1.00000000000000)*(cj5)*(r00)*(sj3)))+(((-1.00000000000000)*(cj3)*(cj5)*(r20)*(sj4))));
        new_r01=((((-1.00000000000000)*(cj5)*(r01)*(sj3)))+(((-1.00000000000000)*(cj3)*(cj5)*(r21)*(sj4)))+(((-1.00000000000000)*(cj3)*(r01)*(sj4)*(sj5)))+(((cj3)*(cj4)*(r11)))+(((r21)*(sj3)*(sj5))));
        new_r02=((((-1.00000000000000)*(cj5)*(r02)*(sj3)))+(((-1.00000000000000)*(cj3)*(r02)*(sj4)*(sj5)))+(((-1.00000000000000)*(cj3)*(cj5)*(r22)*(sj4)))+(((cj3)*(cj4)*(r12)))+(((r22)*(sj3)*(sj5))));
        new_r10=((((cj4)*(cj5)*(r20)))+(((r10)*(sj4)))+(((cj4)*(r00)*(sj5))));
        new_r11=((((cj4)*(cj5)*(r21)))+(((cj4)*(r01)*(sj5)))+(((r11)*(sj4))));
        new_r12=((((r12)*(sj4)))+(((cj4)*(r02)*(sj5)))+(((cj4)*(cj5)*(r22))));
        new_r20=((((cj3)*(cj5)*(r00)))+(((-1.00000000000000)*(cj3)*(r20)*(sj5)))+(((-1.00000000000000)*(cj5)*(r20)*(sj3)*(sj4)))+(((-1.00000000000000)*(r00)*(sj3)*(sj4)*(sj5)))+(((cj4)*(r10)*(sj3))));
        new_r21=((((cj4)*(r11)*(sj3)))+(((-1.00000000000000)*(cj5)*(r21)*(sj3)*(sj4)))+(((-1.00000000000000)*(cj3)*(r21)*(sj5)))+(((cj3)*(cj5)*(r01)))+(((-1.00000000000000)*(r01)*(sj3)*(sj4)*(sj5))));
        new_r22=((((-1.00000000000000)*(cj3)*(r22)*(sj5)))+(((cj4)*(r12)*(sj3)))+(((-1.00000000000000)*(r02)*(sj3)*(sj4)*(sj5)))+(((-1.00000000000000)*(cj5)*(r22)*(sj3)*(sj4)))+(((cj3)*(cj5)*(r02))));
        {
        IKReal j1eval;
        j1eval=new_r21;
        if( j1eval >= 0.999990 && j1eval <= 1.000010 )
        {
            {
            j1 = -1.570796; sj1 = -1.000000; cj1 = 0.000000;
            IKReal j0mul = 1;
            j0=0;
            IKReal j2mul=1.00000000000000;
            j2=((-1.00000000000000)*(IKatan2(new_r10, new_r00)));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[0].fmul = j0mul;
            solution.basesol[0].freeind = 0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[2].fmul = j2mul;
            solution.basesol[2].freeind = 0;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.vfree.resize(1);
            solution.vfree[0] = 0;
            }
            
        } else
        if( j1eval >= -1.000010 && j1eval <= -0.999990 )
        {
            {
            j1 = 1.570796; sj1 = 1.000000; cj1 = 0.000000;
            IKReal j0mul = 1;
            j0=0;
            IKReal j2mul=-1.00000000000000;
            j2=((((-1.00000000000000)*(IKatan2(new_r02, ((-1.00000000000000)*(new_r12))))))+(((-1.00000000000000)*(3.14159265358979))));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[0].fmul = j0mul;
            solution.basesol[0].freeind = 0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[2].fmul = j2mul;
            solution.basesol[2].freeind = 0;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.vfree.resize(1);
            solution.vfree[0] = 0;
            }
            
        } else
        {
            {
            IKReal j0array[2], cj0array[2], sj0array[2];
            bool j0valid[2]={false};
            j0array[0]=IKatan2(((-1.00000000000000)*(new_r20)), new_r22);
            sj0array[0]=IKsin(j0array[0]);
            cj0array[0]=IKcos(j0array[0]);
            j0array[1] = j0array[0] > 0 ? j0array[0]-IKPI : j0array[0]+IKPI;
            sj0array[1] = -sj0array[0];
            cj0array[1] = -cj0array[0];
            if( j0array[0] > IKPI )
                j0array[0]-=IK2PI;
            else if( j0array[0] < -IKPI )
                j0array[0]+=IK2PI;
            j0valid[0] = true;
            if( j0array[1] > IKPI )
                j0array[1]-=IK2PI;
            else if( j0array[1] < -IKPI )
                j0array[1]+=IK2PI;
            j0valid[1] = true;
            if( j0valid[0] && j0valid[1] && IKabs(cj0array[0]-cj0array[1]) < 0.0001 && IKabs(sj0array[0]-sj0array[1]) < 0.0001 )
                j0valid[1]=false;
            for(int ij0 = 0; ij0 < 2; ++ij0)
            {
            if( !j0valid[ij0] ) { continue; }
            j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
            
            {
            IKReal j1array[1], cj1array[1], sj1array[1];
            bool j1valid[1]={false};
            j1array[0]=IKatan2(((-1.00000000000000)*(new_r21)), ((new_r22)*(((IKabs(cj0) != 0)?((IKReal)1/(cj0)):(IKReal)1.0e30))));
            sj1array[0]=IKsin(j1array[0]);
            cj1array[0]=IKcos(j1array[0]);
            if( j1array[0] > IKPI )
                j1array[0]-=IK2PI;
            else if( j1array[0] < -IKPI )
                j1array[0]+=IK2PI;
            j1valid[0] = true;
            for(int ij1 = 0; ij1 < 1; ++ij1)
            {
            if( !j1valid[ij1] ) { continue; }
            j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
            
            {
            IKReal j2array[1], cj2array[1], sj2array[1];
            bool j2valid[1]={false};
            IKReal x300=((IKabs(cj1) != 0)?((IKReal)1/(cj1)):(IKReal)1.0e30);
            j2array[0]=IKatan2(((new_r01)*(x300)), ((new_r11)*(x300)));
            sj2array[0]=IKsin(j2array[0]);
            cj2array[0]=IKcos(j2array[0]);
            if( j2array[0] > IKPI )
                j2array[0]-=IK2PI;
            else if( j2array[0] < -IKPI )
                j2array[0]+=IK2PI;
            j2valid[0] = true;
            for(int ij2 = 0; ij2 < 1; ++ij2)
            {
            if( !j2valid[ij2] ) { continue; }
            j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
            
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.vfree.resize(0);
            }
            }
            }
            }
            }
            }
            
        }
        }
        }
        }
        }
        }
        }
        }
        
    } else
    {
        if( 1 )
        {
        {
        IKReal j3array[2], cj3array[2], sj3array[2];
        bool j3valid[2]={false};
        cj3array[0]=((-0.972903225803748)+(((-1.41935483871202)*(cj6)*(pz)))+(((21.5053763440810)*((py)*(py))))+(((21.5053763440810)*((pz)*(pz))))+(((1.41935483871202)*(px)*(sj6)))+(((0.602150537633861)*(pz)*(sj6)))+(((0.602150537633861)*(cj6)*(px)))+(((21.5053763440810)*((px)*(px)))));
        if( cj3array[0] >= -1.0001 && cj3array[0] <= 1.0001 )
        {
            j3valid[0] = j3valid[1] = true;
            j3array[0] = IKacos(cj3array[0]);
            sj3array[0] = IKsin(j3array[0]);
            cj3array[1] = cj3array[0];
            j3array[1] = -j3array[0];
            sj3array[1] = -sj3array[0];
        }
        else if( isnan(cj3array[0]) )
        {
            // probably any value will work
            j3valid[0] = true;
            cj3array[0] = 1; sj3array[0] = 0; j3array[0] = 0;
        }
        if( j3valid[0] && j3valid[1] && IKabs(cj3array[0]-cj3array[1]) < 0.0001 && IKabs(sj3array[0]-sj3array[1]) < 0.0001 )
            j3valid[1]=false;
        for(int ij3 = 0; ij3 < 2; ++ij3)
        {
        if( !j3valid[ij3] ) { continue; }
        j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
        
        {
        IKReal evalcond[3]; int numsolutionsj4 = 0;
        IKReal j4array[2], cj4array[2], sj4array[2];
        bool j4valid[2]={false};
            {
            evalcond[0]=sj3;
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                cj4array[0]=((6.66666666668607)*(py)*(((IKabs(sj3) != 0)?((IKReal)1/(sj3)):(IKReal)1.0e30)));
                if( cj4array[0] >= -1.0001 && cj4array[0] <= 1.0001 )
                {
                    j4valid[0] = j4valid[1] = true;
                    j4array[0] = IKacos(cj4array[0]);
                    sj4array[0] = IKsin(j4array[0]);
                    cj4array[1] = cj4array[0];
                    j4array[1] = -j4array[0];
                    sj4array[1] = -sj4array[0];
                }
                else if( isnan(cj4array[0]) )
                {
                    // probably any value will work
                    j4valid[0] = true;
                    cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
                }
                numsolutionsj4 = 2;
                
            } else
            {
            IKReal x301=((0.149999999999636)*(sj3));
            IKReal x302=((((-1.00000000000000)*(py)))+(x301));
            evalcond[0]=x302;
            evalcond[1]=((0.0452399999999216)+(((-0.0280000000000200)*(pz)*(sj6)))+(((-0.0280000000000200)*(cj6)*(px)))+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((px)*(px))))+(((0.0659999999998035)*(cj6)*(pz)))+(((-0.0659999999998035)*(px)*(sj6)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x302;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=0;
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            IKReal x303=((-1.00000000000000)*(py));
            evalcond[0]=x303;
            evalcond[1]=((0.0452399999999216)+(((-0.0280000000000200)*(pz)*(sj6)))+(((-0.0280000000000200)*(cj6)*(px)))+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((px)*(px))))+(((0.0659999999998035)*(cj6)*(pz)))+(((-0.0659999999998035)*(px)*(sj6)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x303;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=((0.500000000000000)*(3.14159265358979));
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            IKReal x304=((0.149999999999636)*(sj3));
            IKReal x305=((x304)+(py));
            evalcond[0]=((-1.00000000000000)*(x305));
            evalcond[1]=((0.0452399999999216)+(((-0.0280000000000200)*(pz)*(sj6)))+(((-0.0280000000000200)*(cj6)*(px)))+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((px)*(px))))+(((0.0659999999998035)*(cj6)*(pz)))+(((-0.0659999999998035)*(px)*(sj6)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=x305;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=3.14159265358979;
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
            evalcond[0]=((-1.00000000000000)*(py));
            evalcond[1]=((0.0452399999999216)+(((-0.0280000000000200)*(pz)*(sj6)))+(((-0.0280000000000200)*(cj6)*(px)))+(((0.0464999999999236)*(cj3)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((px)*(px))))+(((0.0659999999998035)*(cj6)*(pz)))+(((-0.0659999999998035)*(px)*(sj6)))+(((-1.00000000000000)*((pz)*(pz)))));
            evalcond[2]=py;
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  )
            {
                j4array[0]=((-0.500000000000000)*(3.14159265358979));
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                numsolutionsj4 = 1;
                
            } else
            {
                continue;
            }
            }
            }
            }
            }
            }
            if( numsolutionsj4 == 0 ) { continue; }
            if( j4valid[0] && j4valid[1] && IKabs(cj4array[0]-cj4array[1]) < 0.0001 && IKabs(sj4array[0]-sj4array[1]) < 0.0001 )
            j4valid[1]=false;
        for(int ij4 = 0; ij4 < numsolutionsj4; ++ij4)
        {
        if( !j4valid[ij4] ) { continue; }
        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
        
        {
        IKReal evalcond[16]; int numsolutionsj5 = 0;
        IKReal j5array[1], cj5array[1], sj5array[1];
        bool j5valid[1]={false};
            {
            evalcond[0]=((((-0.0224999999998090)*((sj3)*(sj3))*((sj4)*(sj4))))+(((-1.00000000000000)*((((0.155000000000655)+(((0.149999999999636)*(cj3)))))*(((0.155000000000655)+(((0.149999999999636)*(cj3)))))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x306=((0.149999999999636)*(cj3));
                IKReal x307=((0.155000000000655)+(x306));
                IKReal x308=((px)*(sj6));
                IKReal x309=((0.0329999999999018)+(x308));
                IKReal x310=((cj6)*(pz));
                IKReal x311=((x309)+(((-1.00000000000000)*(x310))));
                IKReal x312=((cj6)*(px));
                IKReal x313=((pz)*(sj6));
                IKReal x314=((0.0140000000000100)+(x313)+(x312));
                IKReal x315=(x307)*(x307);
                IKReal x316=(sj3)*(sj3);
                IKReal x317=(sj4)*(sj4);
                IKReal x318=((0.0224999999998090)*(x316)*(x317));
                IKReal x319=((x318)+(x315));
                IKReal x320=((-1.00000000000000)*(x319));
                IKReal x321=((IKabs(x320) != 0)?((IKReal)1/(x320)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x321)*(((((-1.00000000000000)*(x307)*(x314)))+(((-0.149999999999636)*(sj3)*(sj4)*(x311)))))), ((x321)*(((((-0.149999999999636)*(sj3)*(sj4)*(x314)))+(((x307)*(x311)))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x322=IKcos(j6);
            IKReal x323=IKsin(j6);
            evalcond[0]=((((((0.155000000000655)+(((0.149999999999636)*(cj3)))))*(((0.0329999999999018)+(((px)*(x323)))+(((-1.00000000000000)*(pz)*(x322)))))))+(((0.149999999999636)*(sj3)*(sj4)*(((0.0140000000000100)+(((pz)*(x323)))+(((px)*(x322))))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x324=((cj6)*(px));
                IKReal x325=((pz)*(sj6));
                IKReal x326=((0.0140000000000100)+(x324)+(x325));
                IKReal x327=((px)*(sj6));
                IKReal x328=((0.0329999999999018)+(x327));
                IKReal x329=((cj6)*(pz));
                IKReal x330=((((-1.00000000000000)*(x329)))+(x328));
                IKReal x331=((0.149999999999636)*(cj3));
                IKReal x332=((0.155000000000655)+(x331));
                IKReal x333=((x330)*(x332));
                IKReal x334=((0.149999999999636)*(sj3)*(sj4)*(x326));
                IKReal x335=((x333)+(x334));
                IKReal x336=((IKabs(x335) != 0)?((IKReal)1/(x335)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x336)*(((((x326)*(x330)))+(((0.149999999999636)*(sj3)*(sj4)*(x332)))))), ((x336)*(((((-1.00000000000000)*((x330)*(x330))))+(((0.0224999999998090)*((sj3)*(sj3))*((sj4)*(sj4))))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x337=IKsin(j6);
            IKReal x338=IKcos(j6);
            evalcond[0]=((((-0.149999999999636)*(sj3)*(sj4)*(((0.0329999999999018)+(((px)*(x337)))+(((-1.00000000000000)*(pz)*(x338)))))))+(((((0.155000000000655)+(((0.149999999999636)*(cj3)))))*(((0.0140000000000100)+(((px)*(x338)))+(((pz)*(x337))))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x339=((0.149999999999636)*(cj3));
                IKReal x340=((0.155000000000655)+(x339));
                IKReal x341=((px)*(sj6));
                IKReal x342=((0.0329999999999018)+(x341));
                IKReal x343=((cj6)*(pz));
                IKReal x344=((((-1.00000000000000)*(x343)))+(x342));
                IKReal x345=((cj6)*(px));
                IKReal x346=((pz)*(sj6));
                IKReal x347=((0.0140000000000100)+(x346)+(x345));
                IKReal x348=((x340)*(x347));
                IKReal x349=((0.149999999999636)*(sj3)*(sj4)*(x344));
                IKReal x350=((x348)+(((-1.00000000000000)*(x349))));
                IKReal x351=((IKabs(x350) != 0)?((IKReal)1/(x350)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x351)*((((x340)*(x340))+(((-1.00000000000000)*((x344)*(x344))))))), ((x351)*(((((0.149999999999636)*(sj3)*(sj4)*(x340)))+(((-1.00000000000000)*(x344)*(x347)))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x352=IKcos(j6);
            IKReal x353=IKsin(j6);
            evalcond[0]=((((0.149999999999636)*(sj3)*(sj4)*(((0.0329999999999018)+(((px)*(x353)))+(((-1.00000000000000)*(pz)*(x352)))))))+(((-1.00000000000000)*(((0.155000000000655)+(((0.149999999999636)*(cj3)))))*(((0.0140000000000100)+(((pz)*(x353)))+(((px)*(x352))))))));
            if( IKabs(evalcond[0]) > 0.000010  )
            {
                IKReal x354=((cj6)*(px));
                IKReal x355=((pz)*(sj6));
                IKReal x356=((0.0140000000000100)+(x355)+(x354));
                IKReal x357=((px)*(sj6));
                IKReal x358=((0.0329999999999018)+(x357));
                IKReal x359=((cj6)*(pz));
                IKReal x360=((((-1.00000000000000)*(x359)))+(x358));
                IKReal x361=((0.149999999999636)*(cj3));
                IKReal x362=((0.155000000000655)+(x361));
                IKReal x363=((0.149999999999636)*(sj3)*(sj4)*(x360));
                IKReal x364=((x356)*(x362));
                IKReal x365=((x363)+(((-1.00000000000000)*(x364))));
                IKReal x366=((IKabs(x365) != 0)?((IKReal)1/(x365)):(IKReal)1.0e30);
                j5array[0]=IKatan2(((x366)*(((((-1.00000000000000)*((x356)*(x356))))+(((0.0224999999998090)*((sj3)*(sj3))*((sj4)*(sj4))))))), ((x366)*(((((x356)*(x360)))+(((-0.149999999999636)*(sj3)*(sj4)*(x362)))))));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x367=((px)*(sj6));
            IKReal x368=((0.149999999999636)*(cj3));
            IKReal x369=((0.188000000000102)+(x368)+(x367));
            IKReal x370=((cj6)*(pz));
            IKReal x371=((((-1.00000000000000)*(x370)))+(x369));
            IKReal x372=((0.149999999999636)*(sj3)*(sj4));
            IKReal x373=((-0.0140000000000100)+(x372));
            IKReal x374=((cj6)*(px));
            IKReal x375=((pz)*(sj6));
            IKReal x376=((x375)+(x374));
            IKReal x377=((x373)+(((-1.00000000000000)*(x376))));
            IKReal x378=(px)*(px);
            IKReal x379=(pz)*(pz);
            IKReal x380=(py)*(py);
            IKReal x381=((0.0280000000000200)*(x374));
            IKReal x382=((0.0280000000000200)*(x375));
            IKReal x383=((0.376000000000204)*(x368));
            IKReal x384=((0.0580399999998917)+(x383));
            IKReal x385=((0.0279999999997926)*(x372));
            IKReal x386=((0.376000000000204)*(x370));
            IKReal x387=((0.376000000000204)*(x367));
            IKReal x388=((x387)+(x382)+(x380)+(x381)+(x379)+(x378));
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=x371;
            evalcond[2]=x377;
            evalcond[3]=((0.0452399999999216)+(((0.175531914894236)*(x386)))+(((-1.00000000000000)*(x380)))+(((-1.00000000000000)*(x381)))+(((-0.175531914894236)*(x387)))+(((-1.00000000000000)*(x382)))+(((-1.00000000000000)*(x379)))+(((0.824468085113040)*(x383)))+(((-1.00000000000000)*(x378))));
            evalcond[4]=x371;
            evalcond[5]=x377;
            evalcond[6]=((((35.7142857140861)*(cj4)*(x382)))+(((-1.00000000000000)*(py)*(sj4)))+(((0.0140000000000100)*(cj4)))+(((35.7142857140861)*(cj4)*(x381))));
            evalcond[7]=((((-1.00000000000000)*(x385)))+(x384)+(((-1.00000000000000)*(x380)))+(((-1.00000000000000)*(x379)))+(((-1.00000000000000)*(x378))));
            evalcond[8]=((((-35.7142857140861)*(sj4)*(x381)))+(((-35.7142857140861)*(sj4)*(x382)))+(((-1.00000000000000)*(cj4)*(py)))+(((-0.0140000000000100)*(sj4)))+(((0.149999999999636)*(sj3))));
            evalcond[9]=((((-1.00000000000000)*(px)))+(((-0.0140000000000100)*(cj6)))+(((-0.188000000000102)*(sj6)))+(((-2.65957446809625)*(sj6)*(x383)))+(((35.7142857145518)*(cj6)*(x385))));
            evalcond[10]=((((-1.00000000000000)*(pz)))+(((2.65957446809625)*(cj6)*(x383)))+(((-0.0140000000000100)*(sj6)))+(((0.188000000000102)*(cj6)))+(((35.7142857145518)*(sj6)*(x385))));
            evalcond[11]=((-0.0130400000003874)+(x386)+(((-1.00000000000000)*(x388))));
            evalcond[12]=((((2.65957446809625)*(sj3)*(x386)))+(((-0.248226950356184)*(sj4)*(x383)))+(((-0.188000000000102)*(sj3)))+(((-2.65957446809625)*(sj3)*(x387)))+(((-17.7304964540526)*(cj4)*(py)*(x383)))+(((-633.232016213238)*(sj4)*(x381)*(x383)))+(((-633.232016213238)*(sj4)*(x382)*(x383))));
            evalcond[13]=((0.149999999999636)+(((-8503.40136063099)*(x382)*(x385)))+(((-3.33333333334303)*(x385)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((-47.1555756758898)*(x383)*(x386)))+(((3.33333333337214)*(x383)))+(((-8503.40136063099)*(x381)*(x385)))+(((47.1555756758898)*(x383)*(x387))));
            evalcond[14]=((((-1.00000000000000)*(x385)))+(x384)+(((-1.00000000000000)*(x380)))+(((-2.27373675443232e-13)*((cj6)*(cj6))))+(((-1.00000000000000)*(x379)))+(((-1.00000000000000)*(x378))));
            evalcond[15]=((-0.0130400000001600)+(x386)+(((-1.00000000000000)*(x388))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  && IKabs(evalcond[12]) <= 0.000010  && IKabs(evalcond[13]) <= 0.000010  && IKabs(evalcond[14]) <= 0.000010  && IKabs(evalcond[15]) <= 0.000010  )
            {
                j5array[0]=0;
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x389=((0.149999999999636)*(cj3));
            IKReal x390=((0.141000000001441)+(x389));
            IKReal x391=((cj6)*(px));
            IKReal x392=((pz)*(sj6));
            IKReal x393=((x391)+(x392));
            IKReal x394=((((-1.00000000000000)*(x393)))+(x390));
            IKReal x395=((cj6)*(pz));
            IKReal x396=((0.149999999999636)*(sj3)*(sj4));
            IKReal x397=((px)*(sj6));
            IKReal x398=(px)*(px);
            IKReal x399=(pz)*(pz);
            IKReal x400=(py)*(py);
            IKReal x401=((0.0659999999998035)*(x395));
            IKReal x402=((0.0659999999998035)*(x397));
            IKReal x403=((0.282000000002881)*(x389));
            IKReal x404=(sj3)*(sj3);
            IKReal x405=(cj4)*(cj4);
            IKReal x406=(cj3)*(cj3);
            IKReal x407=(cj6)*(cj6);
            IKReal x408=((0.0659999999998035)*(x396));
            IKReal x409=((0.282000000002881)*(x392));
            IKReal x410=((0.282000000002881)*(x391));
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((0.0329999999999018)+(((-1.00000000000000)*(x395)))+(((-1.00000000000000)*(x396)))+(x397));
            evalcond[2]=x394;
            evalcond[3]=((0.0452399999999216)+(((-1.00000000000000)*(x399)))+(((1.09929078014102)*(x403)))+(((-0.0992907801410183)*(x409)))+(((-0.0992907801410183)*(x410)))+(((-1.00000000000000)*(x402)))+(((-1.00000000000000)*(x398)))+(x401)+(((-1.00000000000000)*(x400))));
            evalcond[4]=x394;
            evalcond[5]=((-0.0329999999999018)+(((-1.00000000000000)*(x397)))+(x395)+(x396));
            evalcond[6]=((((-1.00000000000000)*(py)*(sj4)))+(((0.0329999999999018)*(cj4)))+(((15.1515151516069)*(cj4)*(x402)))+(((-15.1515151516069)*(cj4)*(x401))));
            evalcond[7]=((0.0434700000000703)+(((-1.00000000000000)*(x399)))+(((-1.00000000000000)*(x408)))+(((-1.00000000000000)*(x398)))+(x403)+(((-1.00000000000000)*(x400))));
            evalcond[8]=((((-15.1515151516069)*(sj4)*(x402)))+(((-1.00000000000000)*(cj4)*(py)))+(((0.149999999999636)*(sj3)))+(((15.1515151516069)*(sj4)*(x401)))+(((-0.0329999999999018)*(sj4))));
            evalcond[9]=((((-1.00000000000000)*(px)))+(((0.141000000001441)*(cj6)))+(((-0.0329999999999018)*(sj6)))+(((3.54609929074650)*(cj6)*(x403)))+(((15.1515151516069)*(sj6)*(x408))));
            evalcond[10]=((((0.0329999999999018)*(cj6)))+(((-1.00000000000000)*(pz)))+(((0.141000000001441)*(sj6)))+(((-15.1515151516069)*(cj6)*(x408)))+(((3.54609929074650)*(sj6)*(x403))));
            evalcond[11]=((0.00152999999960457)+(((-1.00000000000000)*(x399)))+(((-1.00000000000000)*(x402)))+(((-1.00000000000000)*(x398)))+(x401)+(x409)+(((-1.00000000000000)*(x400)))+(x410));
            evalcond[12]=((((-358.191847551614)*(sj4)*(x402)*(x403)))+(((3.54609929074650)*(sj3)*(x410)))+(((358.191847551614)*(sj4)*(x401)*(x403)))+(((-0.780141843963065)*(sj4)*(x403)))+(((-0.141000000001441)*(sj3)))+(((3.54609929074650)*(sj3)*(x409)))+(((-23.6406619383488)*(cj4)*(py)*(x403))));
            evalcond[13]=((0.149999999999636)+(((-83.8321345327422)*(x403)*(x410)))+(((3.33333333334303)*(x403)))+(((-83.8321345327422)*(x403)*(x409)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((1530.45607593656)*(x401)*(x408)))+(((-1530.45607593656)*(x402)*(x408)))+(((-3.33333333334303)*(x408))));
            evalcond[14]=((0.0209700000002613)+(((0.0224999999998090)*(x404)*(x405)*(x407)))+(((0.0224999999998090)*(x404)))+(((-0.0224999999998090)*(x404)*(x405)))+(((-1.00000000000000)*(x399)))+(((0.0224999999998090)*(x407)))+(((-0.0224999999998090)*(x404)*(x407)))+(((-0.0224999999998090)*(x406)*(x407)))+(((-1.00000000000000)*(x408)))+(((-0.0224999999998090)*(x405)*(x407)))+(((-1.00000000000000)*(x398)))+(x403)+(((-1.00000000000000)*(x400)))+(((0.0224999999998090)*(x406)))+(((0.0224999999998090)*(x405)))+(((0.0224999999998090)*(x405)*(x406)*(x407)))+(((-0.0224999999998090)*(x405)*(x406))));
            evalcond[15]=((0.00152999999983194)+(((-1.00000000000000)*(x398)*(x405)*(x406)*(x407)))+(((x398)*(x404)*(x407)))+(((-1.00000000000000)*(x399)))+(((x398)*(x405)*(x407)))+(((x398)*(x404)*(x405)))+(((-1.00000000000000)*(x402)))+(((-1.00000000000000)*(x398)*(x404)*(x405)*(x407)))+(((-1.00000000000000)*(x398)*(x404)))+(((-1.00000000000000)*(x398)*(x407)))+(((x398)*(x405)*(x406)))+(x401)+(x409)+(((-1.00000000000000)*(x400)))+(((-1.00000000000000)*(x398)*(x405)))+(((x398)*(x406)*(x407)))+(x410)+(((-1.00000000000000)*(x398)*(x406))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  && IKabs(evalcond[12]) <= 0.000010  && IKabs(evalcond[13]) <= 0.000010  && IKabs(evalcond[14]) <= 0.000010  && IKabs(evalcond[15]) <= 0.000010  )
            {
                j5array[0]=((0.500000000000000)*(3.14159265358979));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x411=((cj6)*(pz));
            IKReal x412=((0.149999999999636)*(cj3));
            IKReal x413=((px)*(sj6));
            IKReal x414=((0.149999999999636)*(sj3)*(sj4));
            IKReal x415=((cj6)*(px));
            IKReal x416=((pz)*(sj6));
            IKReal x417=(px)*(px);
            IKReal x418=(pz)*(pz);
            IKReal x419=(py)*(py);
            IKReal x420=((0.0280000000000200)*(x415));
            IKReal x421=((0.0280000000000200)*(x416));
            IKReal x422=((0.0279999999997926)*(x414));
            IKReal x423=((0.244000000000597)*(x412));
            IKReal x424=((0.0375800000001618)+(x423)+(x422));
            IKReal x425=((0.244000000002416)*(x413));
            IKReal x426=((0.244000000002416)*(x411));
            IKReal x427=((x426)+(x421)+(x420)+(x417)+(x418)+(x419));
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=((-0.122000000001208)+(((-1.00000000000000)*(x411)))+(((-1.00000000000000)*(x412)))+(x413));
            evalcond[2]=((-0.0140000000000100)+(((-1.00000000000000)*(x414)))+(((-1.00000000000000)*(x415)))+(((-1.00000000000000)*(x416))));
            evalcond[3]=((0.0452399999999216)+(((0.270491803275945)*(x426)))+(((-1.00000000000000)*(x418)))+(((-1.00000000000000)*(x419)))+(((1.27049180328322)*(x423)))+(((-1.00000000000000)*(x421)))+(((-0.270491803275945)*(x425)))+(((-1.00000000000000)*(x417)))+(((-1.00000000000000)*(x420))));
            evalcond[4]=((0.122000000001208)+(((-1.00000000000000)*(x413)))+(x412)+(x411));
            evalcond[5]=((0.0140000000000100)+(x416)+(x414)+(x415));
            evalcond[6]=((((-1.00000000000000)*(py)*(sj4)))+(((-35.7142857140861)*(cj4)*(x421)))+(((-0.0140000000000100)*(cj4)))+(((-35.7142857140861)*(cj4)*(x420))));
            evalcond[7]=((x424)+(((-1.00000000000000)*(x418)))+(((-1.00000000000000)*(x419)))+(((-1.00000000000000)*(x417))));
            evalcond[8]=((((35.7142857140861)*(sj4)*(x421)))+(((0.0140000000000100)*(sj4)))+(((-1.00000000000000)*(cj4)*(py)))+(((35.7142857140861)*(sj4)*(x420)))+(((0.149999999999636)*(sj3))));
            evalcond[9]=((((-1.00000000000000)*(px)))+(((-0.0140000000000100)*(cj6)))+(((-35.7142857145518)*(cj6)*(x422)))+(((0.122000000001208)*(sj6)))+(((4.09836065571290)*(sj6)*(x423))));
            evalcond[10]=((((-1.00000000000000)*(pz)))+(((-0.0140000000000100)*(sj6)))+(((-35.7142857145518)*(sj6)*(x422)))+(((-4.09836065571290)*(cj6)*(x423)))+(((-0.122000000001208)*(cj6))));
            evalcond[11]=((0.00741999999956988)+(x425)+(((-1.00000000000000)*(x427))));
            evalcond[12]=((((-0.122000000001208)*(sj3)))+(((975.800156123936)*(sj4)*(x421)*(x423)))+(((-27.3224043715745)*(cj4)*(py)*(x423)))+(((4.09836065571290)*(sj3)*(x425)))+(((975.800156123936)*(sj4)*(x420)*(x423)))+(((0.382513661199482)*(sj4)*(x423)))+(((-4.09836065571290)*(sj3)*(x426))));
            evalcond[13]=((0.149999999999636)+(((3.33333333337214)*(x423)))+(((8503.40136063099)*(x421)*(x422)))+(((111.977067096159)*(x423)*(x426)))+(((-111.977067096159)*(x423)*(x425)))+(((8503.40136063099)*(x420)*(x422)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((3.33333333334303)*(x422))));
            evalcond[14]=((x424)+(((-1.00000000000000)*(x418)))+(((-1.00000000000000)*(x419)))+(((-2.27373675443232e-13)*((cj6)*(cj6))))+(((-1.00000000000000)*(x417))));
            evalcond[15]=((0.00741999999979726)+(x425)+(((-1.00000000000000)*(x427))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  && IKabs(evalcond[12]) <= 0.000010  && IKabs(evalcond[13]) <= 0.000010  && IKabs(evalcond[14]) <= 0.000010  && IKabs(evalcond[15]) <= 0.000010  )
            {
                j5array[0]=3.14159265358979;
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
            IKReal x428=((cj6)*(px));
            IKReal x429=((pz)*(sj6));
            IKReal x430=((0.149999999999636)*(cj3));
            IKReal x431=((0.149999999999636)*(sj3)*(sj4));
            IKReal x432=((px)*(sj6));
            IKReal x433=((0.0329999999999018)+(x431)+(x432));
            IKReal x434=((cj6)*(pz));
            IKReal x435=((x433)+(((-1.00000000000000)*(x434))));
            IKReal x436=(px)*(px);
            IKReal x437=(pz)*(pz);
            IKReal x438=(py)*(py);
            IKReal x439=((0.0659999999998035)*(x434));
            IKReal x440=((0.0659999999998035)*(x432));
            IKReal x441=(sj3)*(sj3);
            IKReal x442=((0.337999999999738)*(x430));
            IKReal x443=((0.0659999999998035)*(x431));
            IKReal x444=(cj4)*(cj4);
            IKReal x445=(cj3)*(cj3);
            IKReal x446=(cj6)*(cj6);
            IKReal x447=((0.337999999999738)*(x428));
            IKReal x448=((0.337999999999738)*(x429));
            evalcond[0]=((((-1.00000000000000)*(py)))+(((0.149999999999636)*(cj4)*(sj3))));
            evalcond[1]=x435;
            evalcond[2]=((-0.168999999999869)+(((-1.00000000000000)*(x429)))+(((-1.00000000000000)*(x428)))+(((-1.00000000000000)*(x430))));
            evalcond[3]=((0.0452399999999216)+(((0.917159763324889)*(x442)))+(x439)+(((-0.0828402366869341)*(x448)))+(((-0.0828402366869341)*(x447)))+(((-1.00000000000000)*(x438)))+(((-1.00000000000000)*(x437)))+(((-1.00000000000000)*(x440)))+(((-1.00000000000000)*(x436))));
            evalcond[4]=((0.168999999999869)+(x430)+(x429)+(x428));
            evalcond[5]=x435;
            evalcond[6]=((((-1.00000000000000)*(py)*(sj4)))+(((-15.1515151516069)*(cj4)*(x440)))+(((-0.0329999999999018)*(cj4)))+(((15.1515151516069)*(cj4)*(x439))));
            evalcond[7]=((0.0521499999999833)+(x443)+(x442)+(((-1.00000000000000)*(x438)))+(((-1.00000000000000)*(x437)))+(((-1.00000000000000)*(x436))));
            evalcond[8]=((((0.0329999999999018)*(sj4)))+(((-1.00000000000000)*(cj4)*(py)))+(((15.1515151516069)*(sj4)*(x440)))+(((0.149999999999636)*(sj3)))+(((-15.1515151516069)*(sj4)*(x439))));
            evalcond[9]=((((-2.95857988166972)*(cj6)*(x442)))+(((-1.00000000000000)*(px)))+(((-0.168999999999869)*(cj6)))+(((-15.1515151516069)*(sj6)*(x443)))+(((-0.0329999999999018)*(sj6))));
            evalcond[10]=((((0.0329999999999018)*(cj6)))+(((-1.00000000000000)*(pz)))+(((-2.95857988166972)*(sj6)*(x442)))+(((15.1515151516069)*(cj6)*(x443)))+(((-0.168999999999869)*(sj6))));
            evalcond[11]=((-0.00715000000042210)+(x439)+(((-1.00000000000000)*(x447)))+(((-1.00000000000000)*(x438)))+(((-1.00000000000000)*(x448)))+(((-1.00000000000000)*(x437)))+(((-1.00000000000000)*(x440)))+(((-1.00000000000000)*(x436))));
            evalcond[12]=((((-2.95857988166972)*(sj3)*(x447)))+(((-0.168999999999869)*(sj3)))+(((-19.7238658778369)*(cj4)*(py)*(x442)))+(((-2.95857988166972)*(sj3)*(x448)))+(((-298.846452698112)*(sj4)*(x439)*(x442)))+(((298.846452698112)*(sj4)*(x440)*(x442)))+(((0.650887573967339)*(sj4)*(x442))));
            evalcond[13]=((0.149999999999636)+(((3.33333333334303)*(x442)))+(((3.33333333334303)*(x443)))+(((58.3546327748336)*(x442)*(x447)))+(((1530.45607593656)*(x440)*(x443)))+(((-1.00000000000000)*(cj4)*(py)*(sj3)))+(((58.3546327748336)*(x442)*(x448)))+(((-1530.45607593656)*(x439)*(x443))));
            evalcond[14]=((0.0296500000001743)+(((-0.0224999999998090)*(x441)*(x446)))+(((-0.0224999999998090)*(x444)*(x445)))+(x443)+(x442)+(((0.0224999999998090)*(x441)))+(((0.0224999999998090)*(x444)*(x445)*(x446)))+(((-1.00000000000000)*(x438)))+(((0.0224999999998090)*(x441)*(x444)*(x446)))+(((0.0224999999998090)*(x445)))+(((-0.0224999999998090)*(x444)*(x446)))+(((-1.00000000000000)*(x437)))+(((0.0224999999998090)*(x446)))+(((-0.0224999999998090)*(x441)*(x444)))+(((-1.00000000000000)*(x436)))+(((-0.0224999999998090)*(x445)*(x446)))+(((0.0224999999998090)*(x444))));
            evalcond[15]=((-0.00715000000019472)+(((x436)*(x444)*(x446)))+(((-1.00000000000000)*(x436)*(x444)*(x445)*(x446)))+(((-1.00000000000000)*(x436)*(x441)))+(x439)+(((x436)*(x441)*(x446)))+(((x436)*(x445)*(x446)))+(((-1.00000000000000)*(x447)))+(((-1.00000000000000)*(x436)*(x445)))+(((-1.00000000000000)*(x438)))+(((x436)*(x441)*(x444)))+(((-1.00000000000000)*(x436)*(x446)))+(((-1.00000000000000)*(x448)))+(((-1.00000000000000)*(x437)))+(((x436)*(x444)*(x445)))+(((-1.00000000000000)*(x440)))+(((-1.00000000000000)*(x436)*(x444)))+(((-1.00000000000000)*(x436)*(x441)*(x444)*(x446))));
            if( IKabs(evalcond[0]) <= 0.000010  && IKabs(evalcond[1]) <= 0.000010  && IKabs(evalcond[2]) <= 0.000010  && IKabs(evalcond[3]) <= 0.000010  && IKabs(evalcond[4]) <= 0.000010  && IKabs(evalcond[5]) <= 0.000010  && IKabs(evalcond[6]) <= 0.000010  && IKabs(evalcond[7]) <= 0.000010  && IKabs(evalcond[8]) <= 0.000010  && IKabs(evalcond[9]) <= 0.000010  && IKabs(evalcond[10]) <= 0.000010  && IKabs(evalcond[11]) <= 0.000010  && IKabs(evalcond[12]) <= 0.000010  && IKabs(evalcond[13]) <= 0.000010  && IKabs(evalcond[14]) <= 0.000010  && IKabs(evalcond[15]) <= 0.000010  )
            {
                j5array[0]=((-0.500000000000000)*(3.14159265358979));
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                numsolutionsj5 = 1;
                
            } else
            {
                continue;
            }
            }
            }
            }
            }
            }
            }
            }
            }
            if( numsolutionsj5 == 0 ) { continue; }
            for(int ij5 = 0; ij5 < numsolutionsj5; ++ij5)
        {
        if( !j5valid[ij5] ) { continue; }
        j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
        
        new_r00=((((-1.00000000000000)*(cj5)*(cj6)*(r20)*(sj3)))+(((cj3)*(cj5)*(r20)*(sj4)*(sj6)))+(((-1.00000000000000)*(cj6)*(r00)*(sj3)*(sj5)))+(((cj3)*(cj5)*(cj6)*(r00)*(sj4)))+(((cj3)*(r00)*(sj4)*(sj5)*(sj6)))+(((cj3)*(cj4)*(r10)))+(((cj5)*(r00)*(sj3)*(sj6)))+(((-1.00000000000000)*(r20)*(sj3)*(sj5)*(sj6)))+(((-1.00000000000000)*(cj3)*(cj6)*(r20)*(sj4)*(sj5))));
        new_r01=((((cj3)*(cj5)*(r21)*(sj4)*(sj6)))+(((cj5)*(r01)*(sj3)*(sj6)))+(((cj3)*(cj4)*(r11)))+(((-1.00000000000000)*(cj5)*(cj6)*(r21)*(sj3)))+(((cj3)*(r01)*(sj4)*(sj5)*(sj6)))+(((cj3)*(cj5)*(cj6)*(r01)*(sj4)))+(((-1.00000000000000)*(cj3)*(cj6)*(r21)*(sj4)*(sj5)))+(((-1.00000000000000)*(cj6)*(r01)*(sj3)*(sj5)))+(((-1.00000000000000)*(r21)*(sj3)*(sj5)*(sj6))));
        new_r02=((((cj3)*(r02)*(sj4)*(sj5)*(sj6)))+(((-1.00000000000000)*(r22)*(sj3)*(sj5)*(sj6)))+(((cj3)*(cj5)*(r22)*(sj4)*(sj6)))+(((cj3)*(cj5)*(cj6)*(r02)*(sj4)))+(((cj3)*(cj4)*(r12)))+(((-1.00000000000000)*(cj5)*(cj6)*(r22)*(sj3)))+(((-1.00000000000000)*(cj6)*(r02)*(sj3)*(sj5)))+(((cj5)*(r02)*(sj3)*(sj6)))+(((-1.00000000000000)*(cj3)*(cj6)*(r22)*(sj4)*(sj5))));
        new_r10=((((-1.00000000000000)*(cj4)*(r00)*(sj5)*(sj6)))+(((-1.00000000000000)*(cj4)*(cj5)*(cj6)*(r00)))+(((r10)*(sj4)))+(((-1.00000000000000)*(cj4)*(cj5)*(r20)*(sj6)))+(((cj4)*(cj6)*(r20)*(sj5))));
        new_r11=((((-1.00000000000000)*(cj4)*(r01)*(sj5)*(sj6)))+(((-1.00000000000000)*(cj4)*(cj5)*(r21)*(sj6)))+(((r11)*(sj4)))+(((cj4)*(cj6)*(r21)*(sj5)))+(((-1.00000000000000)*(cj4)*(cj5)*(cj6)*(r01))));
        new_r12=((((r12)*(sj4)))+(((-1.00000000000000)*(cj4)*(cj5)*(r22)*(sj6)))+(((-1.00000000000000)*(cj4)*(cj5)*(cj6)*(r02)))+(((-1.00000000000000)*(cj4)*(r02)*(sj5)*(sj6)))+(((cj4)*(cj6)*(r22)*(sj5))));
        new_r20=((((cj3)*(cj6)*(r00)*(sj5)))+(((cj3)*(r20)*(sj5)*(sj6)))+(((cj3)*(cj5)*(cj6)*(r20)))+(((cj5)*(cj6)*(r00)*(sj3)*(sj4)))+(((-1.00000000000000)*(cj3)*(cj5)*(r00)*(sj6)))+(((-1.00000000000000)*(cj6)*(r20)*(sj3)*(sj4)*(sj5)))+(((cj4)*(r10)*(sj3)))+(((cj5)*(r20)*(sj3)*(sj4)*(sj6)))+(((r00)*(sj3)*(sj4)*(sj5)*(sj6))));
        new_r21=((((cj4)*(r11)*(sj3)))+(((cj5)*(r21)*(sj3)*(sj4)*(sj6)))+(((r01)*(sj3)*(sj4)*(sj5)*(sj6)))+(((cj3)*(r21)*(sj5)*(sj6)))+(((-1.00000000000000)*(cj3)*(cj5)*(r01)*(sj6)))+(((-1.00000000000000)*(cj6)*(r21)*(sj3)*(sj4)*(sj5)))+(((cj3)*(cj6)*(r01)*(sj5)))+(((cj5)*(cj6)*(r01)*(sj3)*(sj4)))+(((cj3)*(cj5)*(cj6)*(r21))));
        new_r22=((((-1.00000000000000)*(cj6)*(r22)*(sj3)*(sj4)*(sj5)))+(((cj4)*(r12)*(sj3)))+(((cj5)*(cj6)*(r02)*(sj3)*(sj4)))+(((-1.00000000000000)*(cj3)*(cj5)*(r02)*(sj6)))+(((cj3)*(cj5)*(cj6)*(r22)))+(((cj3)*(r22)*(sj5)*(sj6)))+(((cj5)*(r22)*(sj3)*(sj4)*(sj6)))+(((r02)*(sj3)*(sj4)*(sj5)*(sj6)))+(((cj3)*(cj6)*(r02)*(sj5))));
        {
        IKReal j1eval;
        j1eval=new_r21;
        if( j1eval >= 0.999990 && j1eval <= 1.000010 )
        {
            {
            j1 = -1.570796; sj1 = -1.000000; cj1 = 0.000000;
            IKReal j0mul = 1;
            j0=0;
            IKReal j2mul=1.00000000000000;
            j2=((-1.00000000000000)*(IKatan2(new_r10, new_r00)));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[0].fmul = j0mul;
            solution.basesol[0].freeind = 0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[2].fmul = j2mul;
            solution.basesol[2].freeind = 0;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.vfree.resize(1);
            solution.vfree[0] = 0;
            }
            
        } else
        if( j1eval >= -1.000010 && j1eval <= -0.999990 )
        {
            {
            j1 = 1.570796; sj1 = 1.000000; cj1 = 0.000000;
            IKReal j0mul = 1;
            j0=0;
            IKReal j2mul=-1.00000000000000;
            j2=((((-1.00000000000000)*(IKatan2(new_r02, ((-1.00000000000000)*(new_r12))))))+(((-1.00000000000000)*(3.14159265358979))));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[0].fmul = j0mul;
            solution.basesol[0].freeind = 0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[2].fmul = j2mul;
            solution.basesol[2].freeind = 0;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.vfree.resize(1);
            solution.vfree[0] = 0;
            }
            
        } else
        {
            {
            IKReal j0array[2], cj0array[2], sj0array[2];
            bool j0valid[2]={false};
            j0array[0]=IKatan2(((-1.00000000000000)*(new_r20)), new_r22);
            sj0array[0]=IKsin(j0array[0]);
            cj0array[0]=IKcos(j0array[0]);
            j0array[1] = j0array[0] > 0 ? j0array[0]-IKPI : j0array[0]+IKPI;
            sj0array[1] = -sj0array[0];
            cj0array[1] = -cj0array[0];
            if( j0array[0] > IKPI )
                j0array[0]-=IK2PI;
            else if( j0array[0] < -IKPI )
                j0array[0]+=IK2PI;
            j0valid[0] = true;
            if( j0array[1] > IKPI )
                j0array[1]-=IK2PI;
            else if( j0array[1] < -IKPI )
                j0array[1]+=IK2PI;
            j0valid[1] = true;
            if( j0valid[0] && j0valid[1] && IKabs(cj0array[0]-cj0array[1]) < 0.0001 && IKabs(sj0array[0]-sj0array[1]) < 0.0001 )
                j0valid[1]=false;
            for(int ij0 = 0; ij0 < 2; ++ij0)
            {
            if( !j0valid[ij0] ) { continue; }
            j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
            
            {
            IKReal evalcond;
                evalcond=IKcos(j0);
                if( IKabs(evalcond) < 0.00001 )
                {
                {
                IKReal j2array[1], cj2array[1], sj2array[1];
                bool j2valid[1]={false};
                IKReal x449=IKsin(j0);
                IKReal x450=((IKabs(x449) != 0)?((IKReal)1/(x449)):(IKReal)1.0e30);
                j2array[0]=IKatan2(((-1.00000000000000)*(new_r12)*(x450)), ((new_r02)*(x450)));
                sj2array[0]=IKsin(j2array[0]);
                cj2array[0]=IKcos(j2array[0]);
                if( j2array[0] > IKPI )
                    j2array[0]-=IK2PI;
                else if( j2array[0] < -IKPI )
                    j2array[0]+=IK2PI;
                j2valid[0] = true;
                for(int ij2 = 0; ij2 < 1; ++ij2)
                {
                if( !j2valid[ij2] ) { continue; }
                j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                
                {
                IKReal j1array[1], cj1array[1], sj1array[1];
                bool j1valid[1]={false};
                j1array[0]=IKatan2(((-1.00000000000000)*(new_r21)), ((-1.00000000000000)*(new_r20)*(((IKabs(IKsin(j0)) != 0)?((IKReal)1/(IKsin(j0))):(IKReal)1.0e30))));
                sj1array[0]=IKsin(j1array[0]);
                cj1array[0]=IKcos(j1array[0]);
                if( j1array[0] > IKPI )
                    j1array[0]-=IK2PI;
                else if( j1array[0] < -IKPI )
                    j1array[0]+=IK2PI;
                j1valid[0] = true;
                for(int ij1 = 0; ij1 < 1; ++ij1)
                {
                if( !j1valid[ij1] ) { continue; }
                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                
                vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                solution.basesol.resize(7);
                solution.basesol[0].foffset = j0;
                solution.basesol[1].foffset = j1;
                solution.basesol[2].foffset = j2;
                solution.basesol[3].foffset = j3;
                solution.basesol[4].foffset = j4;
                solution.basesol[5].foffset = j5;
                solution.basesol[6].foffset = j6;
                solution.vfree.resize(0);
                }
                }
                }
                }
                
            } else
            {
                if( 1 )
                {
                {
                IKReal j1array[1], cj1array[1], sj1array[1];
                bool j1valid[1]={false};
                j1array[0]=IKatan2(((-1.00000000000000)*(new_r21)), ((new_r22)*(((IKabs(cj0) != 0)?((IKReal)1/(cj0)):(IKReal)1.0e30))));
                sj1array[0]=IKsin(j1array[0]);
                cj1array[0]=IKcos(j1array[0]);
                if( j1array[0] > IKPI )
                    j1array[0]-=IK2PI;
                else if( j1array[0] < -IKPI )
                    j1array[0]+=IK2PI;
                j1valid[0] = true;
                for(int ij1 = 0; ij1 < 1; ++ij1)
                {
                if( !j1valid[ij1] ) { continue; }
                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                
                {
                IKReal evalcond;
                    evalcond=IKcos(j1);
                    if( IKabs(evalcond) < 0.00001 )
                    {
                    {
                    IKReal j2array[1], cj2array[1], sj2array[1];
                    bool j2valid[1]={false};
                    IKReal x451=IKsin(j1);
                    IKReal x452=((IKabs(x451) != 0)?((IKReal)1/(x451)):(IKReal)1.0e30);
                    IKReal x453=IKsin(j0);
                    IKReal x454=IKcos(j0);
                    j2array[0]=IKatan2(((((new_r02)*(x452)*(x454)))+(((-1.00000000000000)*(new_r00)*(x452)*(x453)))), ((((new_r02)*(x453)))+(((new_r00)*(x454)))));
                    sj2array[0]=IKsin(j2array[0]);
                    cj2array[0]=IKcos(j2array[0]);
                    if( j2array[0] > IKPI )
                        j2array[0]-=IK2PI;
                    else if( j2array[0] < -IKPI )
                        j2array[0]+=IK2PI;
                    j2valid[0] = true;
                    for(int ij2 = 0; ij2 < 1; ++ij2)
                    {
                    if( !j2valid[ij2] ) { continue; }
                    j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                    
                    vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                    solution.basesol.resize(7);
                    solution.basesol[0].foffset = j0;
                    solution.basesol[1].foffset = j1;
                    solution.basesol[2].foffset = j2;
                    solution.basesol[3].foffset = j3;
                    solution.basesol[4].foffset = j4;
                    solution.basesol[5].foffset = j5;
                    solution.basesol[6].foffset = j6;
                    solution.vfree.resize(0);
                    }
                    }
                    
                } else
                {
                    if( 1 )
                    {
                    {
                    IKReal j2array[1], cj2array[1], sj2array[1];
                    bool j2valid[1]={false};
                    IKReal x455=IKcos(j1);
                    IKReal x456=((IKabs(x455) != 0)?((IKReal)1/(x455)):(IKReal)1.0e30);
                    j2array[0]=IKatan2(((new_r01)*(x456)), ((new_r11)*(x456)));
                    sj2array[0]=IKsin(j2array[0]);
                    cj2array[0]=IKcos(j2array[0]);
                    if( j2array[0] > IKPI )
                        j2array[0]-=IK2PI;
                    else if( j2array[0] < -IKPI )
                        j2array[0]+=IK2PI;
                    j2valid[0] = true;
                    for(int ij2 = 0; ij2 < 1; ++ij2)
                    {
                    if( !j2valid[ij2] ) { continue; }
                    j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                    
                    vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                    solution.basesol.resize(7);
                    solution.basesol[0].foffset = j0;
                    solution.basesol[1].foffset = j1;
                    solution.basesol[2].foffset = j2;
                    solution.basesol[3].foffset = j3;
                    solution.basesol[4].foffset = j4;
                    solution.basesol[5].foffset = j5;
                    solution.basesol[6].foffset = j6;
                    solution.vfree.resize(0);
                    }
                    }
                    
                } else
                {
                }
                }
                }
                }
                }
                
            } else
            {
            }
            }
            }
            }
            }
            
        }
        }
        }
        }
        }
        }
        }
        }
        
    } else
    {
    }
    }
    }
    }
    }
    }
    }
return vsolutions.size()>0;
}

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char** argv)
{
    if( argc != 12+getNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",getNumFreeParameters());
        return 1;
    }

    std::vector<IKSolution> vsolutions;
    std::vector<IKReal> vfree(getNumFreeParameters());
    IKReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ik(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, vsolutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)vsolutions.size());
    std::vector<IKReal> sol(getNumJoints());
    for(size_t i = 0; i < vsolutions.size(); ++i) {
        printf("sol%d (free=%d): ", (int)i, (int)vsolutions[i].GetFree().size());
        std::vector<IKReal> vsolfree(vsolutions[i].GetFree().size());
        vsolutions[i].GetSolution(&sol[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( size_t j = 0; j < sol.size(); ++j)
            printf("%f, ", (float)sol[j]);
        printf("\n");
    }
    return 0;
}

#endif
